Imports System.ComponentModel

''' <summary>
''' Basisklasse fr selbst gezeichnete ScrollBars
''' </summary>
<DefaultEvent("Scroll"), DefaultProperty("Value")> _
Public MustInherit Class CustomScrollBarBase
    Inherits Control

#Region " Private Fields "

    ' Painter, der die ScrollBar zeichnet
    Private m_Painter As ScrollBarPainter

    ' Standard-Eigenschaften einer ScrollBar
    Private m_Minimum As Integer = 0
    Private m_Maximum As Integer = 100
    Private m_Value As Integer = 0
    Private m_SmallChange As Integer = 1
    Private m_LargeChange As Integer = 10

    ' Funktions- bzw. Zeichenflchen der ScrollBar
    Private m_AreaButtonTop As Area
    Private m_AreaButtonBottom As Area
    Private m_AreaLargeScrollTop As Area
    Private m_AreaLargeScrollBottom As Area
    Private m_AreaButtonScroll As Area
    Private m_AreaBackground As Area

    ' Auflistung aller Funtionsflchen
    Private m_Areas As New List(Of Area)

    ' Area, ber der sich die Maus gerade befindet
    Private m_HooveredArea As Area

    ' Orientierung der ScrollBar
    Private m_Orientation As Orientation = Windows.Forms.Orientation.Vertical

    ' Virtuelle Breite (bei vertikaler ScrollBar = Width, bei horizontaler = Height)
    Private m_VirtualWidth As Integer = 0

    ' Mindestgre, die auch beim ndern von MinimumSize im Windows Forms Designer 
    ' nicht unterschritten werden darf
    Private m_MinMinimumSize As Size

    ' Abstand (bei MouseDown) zum linken oberen Punkt der Aktiven Area
    Private m_MovingDistance As Integer

    ' Aktuelle Mouse-Position
    Private m_MousePosition As Point

    ' Flag fr Resize der ScrollBar von dieser Basisklasse aus
    Private m_InitializingSize As Boolean = False

    ' Timer fr normales Scrollen (SmallChange)
    Private WithEvents m_ScrollTimer As New Timer
    ' Timer fr LargeChange
    Private WithEvents m_LargeScrollTimer As New Timer

    ' bentigt fr Scroll-Events
    Private m_ScrollOrientation As ScrollOrientation = ScrollOrientation.VerticalScroll
    Private m_ScrollEventType As ScrollEventType = ScrollEventType.EndScroll

    ' Standard-Event (ist brigens ganz oben auf dieser Seite als Default festgelegt)
    Public Event Scroll(ByVal sender As System.Object, ByVal e As System.Windows.Forms.ScrollEventArgs)

#End Region

#Region " Constructor "

    Public Sub New(ByVal Painter As ScrollBarPainter)
        'Referenz zum Painter bernehmen
        m_Painter = Painter

        ' DoubleBuffer zum Vermindern des Flimmerns aktivieren
        Me.SetStyle(ControlStyles.DoubleBuffer Or ControlStyles.UserPaint _
        Or ControlStyles.AllPaintingInWmPaint, True)

        ' Funktionsflchen neu definieren
        m_AreaButtonTop = New Area(Me)
        m_AreaLargeScrollTop = New Area(Me)
        m_AreaButtonScroll = New Area(Me)
        m_AreaLargeScrollBottom = New Area(Me)
        m_AreaButtonBottom = New Area(Me)
        m_AreaBackground = New Area(Me)

        ' fr verschiedene Methoden in dieser Klasse wird eine Liste mit allen 
        ' Funktionsflchen bentigt
        m_Areas.Add(m_AreaButtonTop)
        m_Areas.Add(m_AreaLargeScrollTop)
        m_Areas.Add(m_AreaButtonScroll)
        m_Areas.Add(m_AreaLargeScrollBottom)
        m_Areas.Add(m_AreaButtonBottom)
        m_Areas.Add(m_AreaBackground)

        ' Timer-Intervalle fr SmallChange und LargeChange festlegen
        m_ScrollTimer.Interval = 50
        m_LargeScrollTimer.Interval = 250

        ' dem Painter dieses ScrollBarSurface zuordnen
        ' und Standard-Werte fr Areas und Control vom Painter bernehmen
        ' Jeder ScrollBarArea wird der entsprechende ScrollBarAreaPainter zugeordnet
        ' (in der Folge erhlt brigens auch der Painter eine Refenrenz zur Area)
        m_AreaButtonTop.Painter = m_Painter.AreaButtonTopPainter
        m_AreaLargeScrollTop.Painter = m_Painter.AreaLargeScrollTopPainter
        m_AreaButtonScroll.Painter = m_Painter.AreaButtonScrollPainter
        m_AreaLargeScrollBottom.Painter = m_Painter.AreaLargeScrollBottomPainter
        m_AreaButtonBottom.Painter = m_Painter.AreaButtonBottomPainter
        m_AreaBackground.Painter = m_Painter.AreaBackgroundPainter

        ' MinimumSize und Size fr die ScrollBar neu ermitteln:
        Dim MinLength As Integer = m_Painter.ButtonTopLength + m_Painter.ButtonBottomLength
        ' Hintergrund-Area ist so lang wie die ScrollBar in seiner Lngen-Ausdehnung werden soll
        m_AreaBackground.Length = Math.Max(MinLength, 80)
        m_VirtualWidth = Painter.MinVirtualWidth

        m_MinMinimumSize = New Size(m_VirtualWidth, MinLength)
        Dim ControlSize As Size = New Size(m_VirtualWidth, m_AreaBackground.Length)

        ' Default-Werte fr Buttons bernehmen
        m_AreaButtonTop.Length = m_Painter.ButtonTopLength
        m_AreaLargeScrollTop.Position = m_AreaButtonTop.Endposition
        m_AreaButtonBottom.Length = m_Painter.ButtonBottomLength
        m_AreaButtonBottom.Position = m_AreaBackground.Length - m_AreaButtonBottom.Length

        ' Positionen und Lngen fr bewegliche Areas ermitteln
        GetScrollButton()

        ' Fr alle Areas ein Painter-Update verlangen
        For Each Area As Area In m_Areas
            Area.UpdatePainter()
        Next

        ' Ermittelte neue Werte fr Size und MinimumSize dem Control bergeben 
        m_InitializingSize = True
        MyBase.Size = ControlSize
        MyBase.MinimumSize = m_MinMinimumSize
        m_InitializingSize = False
    End Sub

#End Region

#Region " Properties "

    ''' <summary>
    ''' Orientierung der ScrollBar
    ''' </summary>
    <Description("Orientierung der ScrollBar."), _
    Category("ScrollBarDarstellung")> _
    Property Orientation() As Orientation
        Get
            Return m_Orientation
        End Get
        Set(ByVal value As Orientation)
            ' nur wenn Orientierung sich wirklich ndert
            If m_Orientation <> value Then
                ' neuen Wert fr Orientierung bernehmen
                m_Orientation = value

                ' alte Werte fr Size und MinimumSize festhalten
                Dim OldSize As Size = MyBase.Size


                ' alle Funktionsflchen neu berechnen und ein Update des
                ' Painters verlangen
                For Each Area As Area In m_Areas
                    Area.UpdatePainter()
                Next

                m_MinMinimumSize = New Size(m_MinMinimumSize.Height, m_MinMinimumSize.Width)
                ' MinimumSize vorbergehend auer Kraft setzen
                MyBase.MinimumSize = New Size(0, 0)

                ' Grennderungs-Flag auf True setzen
                m_InitializingSize = True

                ' Size und MinimumSize  fr das Control neu festlegen
                MyBase.Size = New Size(OldSize.Height, OldSize.Width)
                MyBase.MinimumSize = New Size(m_MinMinimumSize.Width, m_MinMinimumSize.Height)

                ' Grennderungs-Flag auf False setzen
                m_InitializingSize = False

                ' Neuzeichnen erzwingen
                Invalidate()
            End If
        End Set
    End Property

    ''' <summary>
    ''' unterer Grenzwert des bildlauffhigen Bereichs
    ''' </summary>
    <Description("Der untere Grenzwert des bildlauffhigen Bereichs."), _
    Category("ScrollBarWerte"), DefaultValue(0)> _
    Property Minimum() As Integer
        Get
            Return m_Minimum
        End Get
        Set(ByVal value As Integer)
            If m_Minimum <> value Then
                m_Minimum = value
                ' Maximum darf nicht kleiner als Minimum sein
                m_Maximum = Math.Max(m_Minimum, m_Maximum)

                ' SmallChange- und LargeChange-Minimalwerte fr Minimum und Maximum ermitteln
                Dim Diff As Integer = m_Maximum - m_Minimum + 1
                m_SmallChange = Math.Min(m_SmallChange, Diff)
                m_LargeChange = Math.Min(m_LargeChange, Diff)

                Dim OldValue As Integer = m_Value
                ' Value darf auch nicht kleiner als Minimum sein
                m_Value = Math.Max(m_Value, m_Minimum)
                ' ScrollButton (Position und Gre) neu ermitteln
                ResetScrollButton()
                ' Wenn notwendig, Scroll-Event
                If OldValue <> m_Value Then
                    RaiseEvent Scroll(Me, New ScrollEventArgs(ScrollEventType.ThumbPosition, OldValue, m_Value))
                End If
            End If
        End Set
    End Property

    ''' <summary>
    ''' oberer Grenzwert des bildlauffhigen Bereichs
    ''' </summary>
    <Description("Der obere Grenzwert des bildlauffhigen Bereichs."), _
    Category("ScrollBarWerte"), DefaultValue(100)> _
    Property Maximum() As Integer
        Get
            Return m_Maximum
        End Get
        Set(ByVal value As Integer)
            If m_Maximum <> value Then

                m_Maximum = value
                ' Minimum darf nicht grer als Maximum sein
                m_Minimum = Math.Min(m_Minimum, m_Maximum)

                ' SmallChange- und LargeChange-Minimalwerte fr Minimum und Maximum ermitteln
                Dim Diff As Integer = m_Maximum - m_Minimum + 1
                m_SmallChange = Math.Min(m_SmallChange, Diff)
                m_LargeChange = Math.Min(m_LargeChange, Diff)

                Dim OldValue As Integer = m_Value

                ' Value darf auch nicht grer als Maximum sein
                m_Value = Math.Min(m_Value, m_Maximum)
                ' ScrollButton (Position und Gre) neu ermitteln und ggf. Neu-Zeichnen erzwingen
                ResetScrollButton()
                ' Wenn notwendig, Scroll-Event
                If OldValue <> m_Value Then
                    RaiseEvent Scroll(Me, New ScrollEventArgs(ScrollEventType.ThumbPosition, OldValue, m_Value))
                End If
            End If
        End Set
    End Property

    ''' <summary>
    ''' Wert, um den sich die Gre des Bildlauffelds ndert, wenn der Benutzer auf einen Bildlaufpfeil klickt
    ''' </summary>
    <Description("Der Wert, um den sich die Gre des Bildlauffelds ndert, wenn der Benutzer auf einen Bildlaufpfeil klickt."), _
    Category("ScrollBarWerte"), DefaultValue(1)> _
    Property SmallChange() As Integer
        Get
            Return m_SmallChange
        End Get
        Set(ByVal value As Integer)
            If m_SmallChange <> value Then
                Dim Diff As Integer = m_Maximum - m_Minimum + 1
                ' SmallChange hat feste Grenzen, die von Minimum und Maximum abhngen
                m_SmallChange = Math.Max(Math.Min(value, Diff), 1)
            End If
        End Set
    End Property

    ''' <summary>
    ''' Wert, um den sich die Gre des Bildlauffelds ndert, wenn der Benutzer auf die Bildlaufleiste klickt
    ''' </summary>
    <Description("Der Wert, um den sich die Gre des Bildlauffelds ndert, wenn der Benutzer auf die Bildlaufleiste klickt."), _
    Category("ScrollBarWerte"), DefaultValue(10)> _
    Property LargeChange() As Integer
        Get
            Return m_LargeChange
        End Get
        Set(ByVal value As Integer)
            If m_LargeChange <> value Then

                Dim Diff As Integer = m_Maximum - m_Minimum + 1
                ' LargeChange hat feste Grenzen, die von Minimum und Maximum abhngen
                m_LargeChange = Math.Max(Math.Min(value, Diff), 1)

                ' die Gre des ScrollButtons hngt auch von LargeChange ab, ggf. Neu-Zeichnen erzwingen
                ResetScrollButton()
            End If
        End Set
    End Property

    ''' <summary>
    ''' Wert, den die Position des Bildlauffelds darstellt
    ''' </summary>
    <Description("Der Wert, den die Position des Bildlauffelds darstellt."), _
    Category("ScrollBarWerte"), DefaultValue(0)> _
    Property Value() As Integer
        Get
            Return m_Value
        End Get
        Set(ByVal value As Integer)
            ' Nur wenn Value sich wirklich gendert hat
            If m_Value <> value Then
                ' Value darf nur innerhalb der Grenzen zwischen Minimum und Maximum liegen
                Dim NewValue As Integer = Math.Min(Math.Max(m_Minimum, value), m_Maximum)
                If m_Value <> NewValue Then
                    Dim OldValue As Integer = m_Value
                    m_Value = NewValue
                    ' ScrollButton (Position und Gre) neu ermitteln und ggf. Neu-Zeichnen erzwingen
                    ResetScrollButton()
                    ' Wenn notwendig, Scroll-Event
                    If OldValue <> m_Value Then
                        RaiseEvent Scroll(Me, New ScrollEventArgs(ScrollEventType.ThumbPosition, OldValue, m_Value))
                    End If
                End If
            End If
        End Set
    End Property


    ''' <summary>
    ''' Hintergrundfarbe
    ''' </summary>
    <Description("Die Hintergrundfarbe."), _
    Category("ScrollBarDarstellung")> _
    Overrides Property BackColor() As Color
        Get
            Return m_Painter.ScrollBarColors.BackColor
        End Get
        Set(ByVal value As Color)
            MyBase.BackColor = value
            m_Painter.ScrollBarColors.BackColor = value
            Invalidate()
        End Set
    End Property

    ''' <summary>
    ''' Vordergrundfarbe
    ''' </summary>
    <Description("Die Vordergrundfarbe der ScrollBarButtons."), _
    Category("ScrollBarDarstellung")> _
    Overrides Property ForeColor() As Color
        Get
            Return m_Painter.ScrollBarColors.ForeColor
        End Get
        Set(ByVal value As Color)
            MyBase.ForeColor = value
            m_Painter.ScrollBarColors.ForeColor = value
            Invalidate()
        End Set
    End Property

    ''' <summary>
    ''' Vordergrundfarbe der ScrollBarButtons bei ausgewhltem Zustand
    ''' </summary>
    <Description("Die Vordergrundfarbe der ScrollBarButtons bei ausgewhltem Zustand."), _
    Category("ScrollBarDarstellung")> _
    Property SelectionColor() As Color
        Get
            Return m_Painter.ScrollBarColors.SelectionColor
        End Get
        Set(ByVal value As Color)
            m_Painter.ScrollBarColors.SelectionColor = value
            Invalidate()
        End Set
    End Property

    ''' <summary>
    ''' Vordergrundfarbe der ScrollBarButtons bei MouseOver-Zustand
    ''' </summary>
    <Description("Die Vordergrundfarbe der ScrollBarButtons bei MouseOver-Zustand."), _
    Category("ScrollBarDarstellung")> _
    Property HooverColor() As Color
        Get
            Return m_Painter.ScrollBarColors.HooverColor
        End Get
        Set(ByVal value As Color)
            m_Painter.ScrollBarColors.HooverColor = value
            Invalidate()
        End Set
    End Property

    Public Overrides Property MinimumSize() As System.Drawing.Size
        Get
            Return MyBase.MinimumSize
        End Get
        Set(ByVal value As System.Drawing.Size)
            ' MinimumSize darf die Werte, die der ScrollBarPainter vorgibt,
            ' nicht unterschreiten (wenigstens die beiden Pfeil-Buttons sollen zu sehen sein)
            Dim NewMinSize As New Size(Math.Max(value.Width, m_MinMinimumSize.Width), Math.Max(value.Height, m_MinMinimumSize.Height))
            MyBase.MinimumSize = NewMinSize
        End Set
    End Property

#End Region

#Region " Methods "

    ''' <summary>
    ''' ermittelt die Lngen der beweglichen Areas nach nderung von Value neu
    ''' </summary>
    Private Sub GetScrollButton()

        ' Freier Bereich zwischen dem Top- und Bottom-Button
        Dim FreeScrollLength As Integer = m_AreaBackground.Length - m_AreaButtonTop.Length - m_AreaButtonBottom.Length

        ' Wenn der freie Bereich kleiner als die ScrollButton-Mindesthhe ist
        If FreeScrollLength < m_Painter.MinButtonScrollLength Then
            ' Lngen aller beweglichen Areas auf Null setzen
            m_AreaButtonScroll.Length = 0
            m_AreaLargeScrollTop.Length = 0
            m_AreaLargeScrollBottom.Length = 0
        Else
            ' maximaler Wert fr LargeChange (bestimmt Grsse des ScrollButtons)
            Dim MaxLargeChange As Single = m_Maximum - m_Minimum + 1

            ' ScrollButton-Hhe
            Dim ButtonHeight As Integer = Math.Max(Math.Round(CSng(m_LargeChange) / MaxLargeChange * CSng(FreeScrollLength)), m_Painter.MinButtonScrollLength)

            ' maximal erreichbarer Wert fr Value
            Dim MaxScrollableValue As Single = m_Maximum - m_LargeChange + 1
            ' neuer sichtbarer Value-Wert des ScrollButtons
            Dim NewVisibleValue As Single = Math.Min(m_Value, MaxScrollableValue)

            ' Integer-Wert, der die Spanne zwischen neu sichtbarem Value-Wert und Minimum enthlt 
            Dim ValuesLength As Single = MaxScrollableValue - m_Minimum
            ' Die Hhe des ScrollButtons steht fest.
            ' Um die Position des ScrollButtons zu ermitteln, wird der brig bleibende
            ' freie Bereich der ScrollBar herangezogen
            Dim ScrollBarLength As Single = FreeScrollLength - ButtonHeight

            ' Die neue Position des Buttons innerhalb des scrollbaren Bereiches
            Dim NewPositionInScrollBar As Integer = 0
            If ValuesLength <> 0.0F Then
                Dim ValuePosition As Single = NewVisibleValue - m_Minimum
                NewPositionInScrollBar = Math.Round(ScrollBarLength * ValuePosition / ValuesLength)
            End If

            ' neue Werte auf die Areas bertragen
            m_AreaLargeScrollTop.Length = NewPositionInScrollBar
            m_AreaButtonScroll.Position = m_AreaLargeScrollTop.Endposition
            m_AreaButtonScroll.Length = ButtonHeight
            m_AreaLargeScrollBottom.Position = m_AreaButtonScroll.Endposition
            m_AreaLargeScrollBottom.Length = m_AreaButtonBottom.Position - m_AreaLargeScrollBottom.Position
        End If

    End Sub

    ''' <summary>
    ''' Passt die Scroll-Button-Position nach nderungen von Properties an und sorgt 
    ''' fr Neu-Zeichnen nur des genderten Rechtecks
    ''' </summary>
    Private Sub ResetScrollButton()
        ' Zeichenrechteck des ScrollButtons festhalten
        Dim OldRect As Rectangle = m_Painter.AreaButtonScrollPainter.PaintRectangle
        ' ScrollButton-Position neu ermitteln und Positionen der beweglichen Areas neu festlegen
        GetScrollButton()

        Dim HasChanges As Boolean = False

        ' nur alle Areas, die gendert wurden, brauchen ein Rechteck- und ein Painter-Update
        For Each Area As Area In m_Areas
            If Area.HasRectangleChanges Then
                Area.UpdatePainter()
                HasChanges = True
            End If
        Next

        If HasChanges Then
            ' Neues Zeichenrechteck des ScrollButtons
            Dim NewRect As Rectangle = m_Painter.AreaButtonScrollPainter.PaintRectangle
            ' Neuzeichnen des Controls erzwingen
            Invalidate(Rectangle.Union(OldRect, NewRect))
        End If
    End Sub

    ''' <summary>
    ''' passt abhngige Areas nach Grennderung des Controls an
    ''' </summary>
    ''' <param name="NewValue">Virtuelle Lnge</param>
    Private Sub SetLength(ByVal NewValue As Integer)
        If m_AreaBackground.Length <> NewValue Then
            ' Lnge der Hintergrund-Area neu festlegen
            m_AreaBackground.Length = NewValue
            ' Position des (bei vertikaler Darstellung) unteren Pfeil-Buttons neu festlegen
            m_AreaButtonBottom.Position = m_AreaBackground.Length - m_AreaButtonBottom.Length
            ' Position und Lnge der beweglichen Areas neu festlegen
            GetScrollButton()
        End If
    End Sub

    ''' <summary>
    ''' passt abhngige Areas nach Grennderung des Controls an
    ''' </summary>
    ''' <param name="NewValue">Virtuelle Breite</param>
    Private Sub SetVirtualWidth(ByVal NewValue As Integer)
        If m_VirtualWidth <> NewValue Then
            ' Alle Areas verwenden dieselbe virtuelle Breite 
            m_VirtualWidth = NewValue
            For Each Area As Area In m_Areas
                ' Alle Areas werden als gendert markiert
                Area.HasRectangleChanges = True
            Next
        End If
    End Sub

    ''' <summary>
    ''' gibt die virtuelle Breite der ScrollBar zurck
    ''' </summary>
    ''' <remarks>fr Orientierung = vertikal: Width
    ''' fr Orientierung = horizontal: height</remarks>
    Public Function VirtualWidth() As Integer
        Return m_VirtualWidth
    End Function

    ''' <summary>
    ''' bewirkt eine nderung der Position des ScrollButtons nach Events durch den ScrollTimer
    ''' und fhrt zum Auslsen des Scroll-Events
    ''' </summary>
    Private Sub ScrollToNewValue(ByVal NewValue As Integer)
        ' nur wenn Value wirklich gendert wird
        If m_Value <> NewValue Then
            ' alten Wert frs Event festhalten
            Dim OldValue As Integer = m_Value
            ' neuen Wert bernehmen
            m_Value = NewValue
            ' ScrollButton neu positionieren und genderte Flche des Controls neu zeichnen
            ResetScrollButton()
            ' Event auslsen
            RaiseEvent Scroll(Me, New ScrollEventArgs(m_ScrollEventType, OldValue, m_Value, m_ScrollOrientation))
        End If
    End Sub

    ''' <summary>
    ''' Scrollen nach Klicken und Festhalten der Maus ber einem Pfeil-Button 
    ''' oder einer LargeChange-Flche
    ''' </summary>
    ''' <remarks>
    ''' fhrt zum Neufestlegen von Value, Auslsen des Scroll-Events und 
    ''' zum Neuzeichnen genderter ControlFlchen nach einem Timer-Event
    ''' </remarks>
    Private Sub ScrollTimer_Tick(ByVal sender As Object, ByVal e As System.EventArgs) Handles m_ScrollTimer.Tick, m_LargeScrollTimer.Tick

        If m_AreaButtonTop.Rectangle.Contains(m_MousePosition) AndAlso m_ScrollEventType = ScrollEventType.SmallDecrement Then
            ' SmallChange mit oberem Pfeil-Button
            If m_Value > m_Minimum Then
                ScrollToNewValue(Math.Max(m_Value - m_SmallChange, m_Minimum))
            End If
        ElseIf m_AreaButtonBottom.Rectangle.Contains(m_MousePosition) AndAlso m_ScrollEventType = ScrollEventType.SmallIncrement Then
            ' SmallChange mit unterem Pfeil-Button
            Dim Maximum As Integer = m_Maximum - m_LargeChange + 1
            If m_Value < Maximum Then
                ScrollToNewValue(Math.Min(m_Value + m_SmallChange, Maximum))
            End If
        ElseIf m_AreaLargeScrollTop.Rectangle.Contains(m_MousePosition) AndAlso m_ScrollEventType = ScrollEventType.LargeDecrement Then
            ' LargeChange mit oberer LargeChange-Flche
            If m_Value > m_Minimum Then
                ScrollToNewValue(Math.Max(m_Value - m_LargeChange, m_Minimum))
            End If
        ElseIf m_AreaLargeScrollBottom.Rectangle.Contains(m_MousePosition) AndAlso m_ScrollEventType = ScrollEventType.LargeIncrement Then
            ' LargeChange mit unterer LargeChange-Flche
            Dim Maximum As Integer = m_Maximum - m_LargeChange + 1
            If m_Value < Maximum Then
                ScrollToNewValue(Math.Min(m_Value + m_LargeChange, Maximum))
            End If
        End If

    End Sub

    Protected Overrides Sub OnPaint(ByVal e As System.Windows.Forms.PaintEventArgs)
        MyBase.OnPaint(e)
        ' der Painter zeichnet die Oberflche des Controls neu
        m_Painter.OnPaint(e)
    End Sub

    Protected Overrides Sub OnResize(ByVal e As System.EventArgs)
        MyBase.OnResize(e)
        ' fhrt bei Grennderung des Controls zum Neupositionieren der Areas

        ' nur reagieren, wenn die Gre des Controls nicht durch diese Basisklasse selbst
        ' gendert wird
        If m_InitializingSize Then Exit Sub

        ' alle Neue Werte fr Size auf die Funktionsflchen bertragen
        If m_Orientation = Windows.Forms.Orientation.Vertical Then
            SetVirtualWidth(Me.Size.Width)
            SetLength(Me.Size.Height)
        Else
            SetVirtualWidth(Me.Size.Height)
            SetLength(Me.Size.Width)
        End If

        ' Alle Funktionsflchen, die wirklich gendert wurden, mssen ein Update des
        ' zugehrigen AreaPainters erreichen
        For Each Area As Area In m_Areas
            If Area.HasRectangleChanges Then
                Area.UpdatePainter()
            End If
        Next

        ' Neuzeichnen fr das ganze Control erzwingen
        Invalidate()
    End Sub

    Protected Overrides Sub OnMouseDown(ByVal e As System.Windows.Forms.MouseEventArgs)
        MyBase.OnMouseDown(e)

        ' wenn der linke Mausbutton bedient wird
        If e.Button = MouseButtons.Left Then
            ' ist zumindestens der Hintergrund selektiert
            m_AreaBackground.Selected = True

            ' wenn sich die Maus gerade ber einer Funktionsflche befindet
            If m_HooveredArea IsNot Nothing Then
                ' ist die aktive Funktionsflche auch selektiert
                m_HooveredArea.Selected = True

                If m_HooveredArea Is m_AreaButtonTop Then
                    ' wenn der obere Pfeil-Button geklickt wurde, wird 
                    ' mit SmallChange nach oben gescrollt
                    If m_Value > m_Minimum Then
                        m_ScrollEventType = ScrollEventType.SmallDecrement
                        ScrollToNewValue(Math.Max(m_Value - m_SmallChange, m_Minimum))
                        m_ScrollTimer.Start()
                    End If
                ElseIf m_HooveredArea Is m_AreaButtonBottom Then
                    ' wenn der untere Pfeil-Button geklickt wurde, wird 
                    ' mit SmallChange nach unten gescrollt
                    Dim Maximum As Integer = m_Maximum - m_LargeChange + 1
                    If m_Value < Maximum Then
                        m_ScrollEventType = ScrollEventType.SmallIncrement
                        ScrollToNewValue(Math.Min(m_Value + m_SmallChange, Maximum))
                        m_ScrollTimer.Start()
                    End If
                ElseIf m_HooveredArea Is m_AreaButtonScroll Then
                    ' wenn der ScrollButton angeklickt wurde, wird 
                    ' wird alles zum Ziehen des Buttons mit der Maus vorbereitet
                    m_ScrollEventType = ScrollEventType.ThumbTrack
                    If m_Orientation = Windows.Forms.Orientation.Vertical Then
                        m_MovingDistance = e.Location.Y - m_AreaButtonScroll.Position
                    Else
                        m_MovingDistance = e.Location.X - m_AreaButtonScroll.Position
                    End If
                ElseIf m_HooveredArea Is m_AreaLargeScrollTop Then
                    ' wenn die obere LargeScroll-Flche geklickt wurde, wird 
                    ' mit LargChange nach oben gescrollt
                    If m_Value > m_Minimum Then
                        m_ScrollEventType = ScrollEventType.LargeDecrement
                        ScrollToNewValue(Math.Max(m_Value - m_LargeChange, m_Minimum))
                        m_LargeScrollTimer.Start()
                    End If
                ElseIf m_HooveredArea Is m_AreaLargeScrollBottom Then
                    ' wenn die untere LargeScroll-Flche geklickt wurde, wird 
                    ' mit LargChange nach unten gescrollt
                    Dim Maximum As Integer = m_Maximum - m_LargeChange + 1
                    If m_Value < Maximum Then
                        m_ScrollEventType = ScrollEventType.LargeIncrement
                        ScrollToNewValue(Math.Min(m_Value + m_LargeChange, Maximum))
                        m_LargeScrollTimer.Start()
                    End If
                End If
            End If

        End If
    End Sub

    Protected Overrides Sub OnMouseUp(ByVal e As System.Windows.Forms.MouseEventArgs)
        MyBase.OnMouseUp(e)

        ' alle ScrollTimer anhalten, falls sie aktiv sind
        m_ScrollTimer.Stop()
        m_LargeScrollTimer.Stop()

        ' Hintergrund- und aktive MouseOver-Area sind nicht mehr selektiert
        ' (bleiben aber gehoovert)
        m_AreaBackground.Selected = False
        If m_HooveredArea IsNot Nothing Then
            m_HooveredArea.Selected = False
        End If
        m_ScrollEventType = ScrollEventType.EndScroll
    End Sub

    Protected Overrides Sub OnMouseMove(ByVal e As System.Windows.Forms.MouseEventArgs)
        MyBase.OnMouseMove(e)
        ' beim Bewegen der Maus ber das Control

        ' wenn kein Button geklickt wurde
        If e.Button = Windows.Forms.MouseButtons.None Then

            ' ist zumindestend der Hintergrund gehoovert
            m_AreaBackground.Hoovered = True

            ' ggf. befindet sich die Maus ber einer Funktionsflche
            Dim NewArea As Area = Nothing
            For Each Area As Area In m_Areas
                If Area.Rectangle.Contains(e.Location) Then
                    If Not Area Is m_AreaBackground Then
                        NewArea = Area
                        Exit For
                    End If
                End If
            Next

            ' Wenn die aktuelle Funktionsflche nicht der neuen gleicht,
            ' wird die neue gehoovert, die alte verliert ihren Hoovered-Status
            If m_HooveredArea IsNot NewArea Then
                If m_HooveredArea IsNot Nothing Then m_HooveredArea.Hoovered = False
                m_HooveredArea = NewArea
                If m_HooveredArea IsNot Nothing Then m_HooveredArea.Hoovered = True
            End If

        ElseIf e.Button = Windows.Forms.MouseButtons.Left AndAlso m_ScrollEventType = ScrollEventType.ThumbTrack Then
            ' wenn der linke Maus-Button gedrckt ist und schon bei MouseDown der ScrollEventType
            ' auf's Ziehen mit der Maus festgelegt wurde, wrd der Button an die neue Position bewegt

            ' NewxValue ist bei vertikaler Orientierung der neue Top-Wert des Buttons,
            ' bei horizontaler Orientierung der neue Left-Wert
            Dim NewxValue As Integer
            If m_Orientation = Windows.Forms.Orientation.Vertical Then
                NewxValue = e.Y - m_MovingDistance
            Else
                NewxValue = e.X - m_MovingDistance
            End If

            ' die neue Position des Scroll-Buttons darf nur innerhalb bestimmter Grenzen liegen
            NewxValue = Math.Max(Math.Min(NewxValue, m_AreaBackground.Length - m_AreaButtonBottom.Length - m_AreaButtonScroll.Length), m_AreaButtonTop.Length)

            ' Wenn sich die Position des Buttons wirklich gendert hat
            If m_AreaButtonTop.Length + m_AreaLargeScrollTop.Length <> NewxValue Then
                ' altes Zeichenrechteck des ScrollButtons festhalten
                Dim OldRect As Rectangle = m_Painter.AreaButtonScrollPainter.PaintRectangle

                ' Positionen und Lngen der beweglichen Funktionsflchen neu festlegen
                m_AreaLargeScrollTop.Length = NewxValue - m_AreaButtonTop.Length
                m_AreaButtonScroll.Position = NewxValue
                m_AreaLargeScrollBottom.Position = m_AreaButtonScroll.Endposition
                m_AreaLargeScrollBottom.Length = m_AreaButtonBottom.Position - m_AreaLargeScrollBottom.Position

                ' Fr alle genderten Funktionsflchen ein Painter-Update durchsetzen
                For Each Area As Area In m_Areas
                    If Area.HasRectangleChanges Then
                        Area.UpdatePainter()
                    End If
                Next

                ' Neues Zeichenrechteck des ScrollButtons
                Dim NewRect As Rectangle = m_Painter.AreaButtonScrollPainter.PaintRectangle
                ' Neuzeichnen nur fr die genderte Flche erzwingen
                Invalidate(Rectangle.Union(OldRect, NewRect))

                ' erst jetzt den neuen Wert fr Value ermitteln
                ' maximal per Maus erreichbarer Wert fr Value der ScrollBar
                Dim MaxScrollableValue As Single = m_Maximum - m_LargeChange + 1
                ' Die Spannbreite zwischen maximal per Maus erreichbarem Wert und Minimum
                Dim ValuesLength As Single = MaxScrollableValue - m_Minimum
                ' der freie Bereich der ScrollBar
                Dim ScrollableLength As Single = m_AreaBackground.Length - m_AreaButtonBottom.Length - m_AreaButtonScroll.Length - m_AreaButtonTop.Length

                ' neuer ermittelter Wert fr Value
                Dim NewValue As Integer
                If ScrollableLength <> 0.0F Then
                    Dim PositionInScrollBar As Single = NewxValue - m_AreaButtonTop.Length
                    NewValue = Math.Round((ValuesLength * PositionInScrollBar) / ScrollableLength) + CSng(m_Minimum)
                Else
                    NewValue = m_Minimum
                End If

                ' wenn der neue Wert fr Value vom alten abweicht
                If m_Value <> NewValue Then
                    ' alten Wert festhalten
                    Dim OldValue As Integer = m_Value
                    ' neuen Wert bernehmen
                    m_Value = NewValue
                    ' Scroll-Event auslsen
                    RaiseEvent Scroll(Me, New ScrollEventArgs(m_ScrollEventType, OldValue, NewValue, m_ScrollOrientation))
                End If
            End If
        Else
            ' in allen anderen Fllen aktuelle Mausposition festhalten (wird gebraucht fr 
            ' Scrollen mit LargeChange und SmallChange, dabei soll Scrollen nur mglich sein, 
            ' wenn sich die Maus gerade ber der aktiven Area befindet)
            m_MousePosition = e.Location
        End If
    End Sub

    Protected Overrides Sub OnMouseEnter(ByVal e As System.EventArgs)
        MyBase.OnMouseEnter(e)
        ' wenn sich die Maus in den Control-Bereich hineinbewegt, wird der Hintergrund gehoovert
        m_AreaBackground.Hoovered = True
    End Sub

    Protected Overrides Sub OnMouseLeave(ByVal e As System.EventArgs)
        MyBase.OnMouseLeave(e)

        ' Wenn die Maus den Control-Bereich verlsst
        ' (passiert auch nach MouseUp bei vorherigem Click auf dieses Control)

        ' die ScrollTimer werden gestoppt
        m_ScrollTimer.Stop()
        m_LargeScrollTimer.Stop()

        ' alle Funktionsflchen werden in den inaktiven Zustand versetzt
        If m_HooveredArea IsNot Nothing Then
            m_HooveredArea.Hoovered = False
            m_HooveredArea = Nothing
        End If
        m_AreaBackground.Hoovered = False
        m_ScrollEventType = ScrollEventType.EndScroll
    End Sub

    Protected Overrides Sub OnEnabledChanged(ByVal e As System.EventArgs)
        MyBase.OnEnabledChanged(e)
        ' Umstellen des Aktivierungs-Zustands
        For Each Area As Area In m_Areas
            Area.UpdatePainter()
        Next
        ' gesamte ScrollBar zum Neuzeichnen zwingen
        Invalidate()
    End Sub

#End Region

End Class
