''' <summary>
''' Eine Instanz dieser Klasse liefert fr jede Area der ScrollBar
''' einen eigenen Painter, um damit die ScrollBar zu zeichnen 
''' </summary>
Public MustInherit Class ScrollBarPainter

#Region " Properties "

#Region " AreaPainters "
    ''' <summary>
    ''' AreaPainter fr den oberen Pfeil-Button
    ''' </summary>
    MustOverride ReadOnly Property AreaButtonTopPainter() As ScrollBarAreaPainter

    ''' <summary>
    ''' AreaPainter fr den unteren Pfeil-Button
    ''' </summary>
    MustOverride ReadOnly Property AreaButtonBottomPainter() As ScrollBarAreaPainter

    ''' <summary>
    ''' AreaPainter fr die obere Scroll-Flche
    ''' </summary>
    MustOverride ReadOnly Property AreaLargeScrollTopPainter() As ScrollBarAreaPainter

    ''' <summary>
    ''' AreaPainter fr die untere Scroll-Flche
    ''' </summary>
    MustOverride ReadOnly Property AreaLargeScrollBottomPainter() As ScrollBarAreaPainter

    ''' <summary>
    ''' AreaPainter fr den Scroll-Button
    ''' </summary>
    MustOverride ReadOnly Property AreaButtonScrollPainter() As ScrollBarAreaPainter

    ''' <summary>
    ''' AreaPainter fr den Hintergrund des Controls
    ''' </summary>
    MustOverride ReadOnly Property AreaBackgroundPainter() As ScrollBarAreaPainter
#End Region

#Region " Default-Werte fr die ScrollBar "

    ''' <summary>
    ''' Lnge des oberen Pfeil-Buttons
    ''' </summary>
    MustOverride ReadOnly Property ButtonTopLength() As Integer

    ''' <summary>
    ''' Lnge des unteren Pfeil-Buttons
    ''' </summary>
    MustOverride ReadOnly Property ButtonBottomLength() As Integer

    ''' <summary>
    ''' Mindestlnge fr den Scroll-Button
    ''' </summary>
    ''' <value>Ein Wert > 0</value>
    MustOverride ReadOnly Property MinButtonScrollLength() As Integer

    ''' <summary>
    ''' Minimum fr Breite
    ''' </summary>
    MustOverride ReadOnly Property MinVirtualWidth() As Integer

    ''' <summary>
    ''' Standard-Farben der ScrollBar
    ''' </summary>
    MustOverride ReadOnly Property ScrollBarColors() As ScrollBarColorCollection
#End Region

#End Region

#Region " Methods "

    ''' <summary>
    ''' zeichnet die ScrollBar neu
    ''' </summary>
    Public Overridable Sub OnPaint(ByVal e As System.Windows.Forms.PaintEventArgs)
        ' Hintergrund wird immer gezeichnet
        AreaBackgroundPainter.OnPaint(e)
        ' nur wenn ein Zeichenrechteck eines AreaPainters innerhalb des 
        ' ClipRectangle liegt (neu zu zeichnende Flche), wird der entsprechende
        ' AreaPainter zum Zeichnen hinzugezogen
        If e.ClipRectangle.IntersectsWith(AreaButtonBottomPainter.PaintRectangle) Then
            AreaButtonBottomPainter.OnPaint(e)
        End If
        If e.ClipRectangle.IntersectsWith(AreaLargeScrollBottomPainter.PaintRectangle) Then
            AreaLargeScrollBottomPainter.OnPaint(e)
        End If
        If AreaButtonScrollPainter.Area.Enabled AndAlso AreaButtonScrollPainter.Area.Length > 0 Then
            If e.ClipRectangle.IntersectsWith(AreaButtonScrollPainter.PaintRectangle) Then
                AreaButtonScrollPainter.OnPaint(e)
            End If
        End If
        If e.ClipRectangle.IntersectsWith(AreaLargeScrollTopPainter.PaintRectangle) Then
            AreaLargeScrollTopPainter.OnPaint(e)
        End If
        If e.ClipRectangle.IntersectsWith(AreaButtonTopPainter.PaintRectangle) Then
            AreaButtonTopPainter.OnPaint(e)
        End If
    End Sub
#End Region

End Class
