Imports System.Drawing.Drawing2D
''' <summary>
''' Eine Sammlung von hufiger bentigten Funktionen
''' </summary>
''' <remarks></remarks>
Public Class SharedFunctions
    ''' <summary>
    ''' Liefert einen GraphicsPath mit abgerundeten Ecken
    ''' </summary>
    ''' <param name="X">Die X-Koordinate der linken oberen Ecke fr das Ziel-Rechteck</param>
    ''' <param name="Y">Die Y-Koordinate der linken oberen Ecke fr das Ziel-Rechteck</param>
    ''' <param name="Width">Die Breite des Ziel-Rechtecks</param>
    ''' <param name="Height">Die Hhe des Ziel-Rechtecks</param>
    ''' <param name="Radius">Radius fr die runden Ecken</param>
    ''' <returns>GraphicsPath</returns>
    Shared Function GetRoundPath(ByVal X As Single, ByVal Y As Single, ByVal Width As Single, ByVal Height As Single, ByVal Radius As Single) As GraphicsPath
        Dim gp As New GraphicsPath
        gp.AddLine(X + Radius, Y, X + Width - (Radius * 2), Y)
        gp.AddArc(X + Width - (Radius * 2), Y, Radius * 2, Radius * 2, 270, 90)
        gp.AddLine(X + Width, Y + Radius, X + Width, Y + Height - (Radius * 2))
        gp.AddArc(X + Width - (Radius * 2), Y + Height - (Radius * 2), Radius * 2, Radius * 2, 0, 90)
        gp.AddLine(X + Width - (Radius * 2), Y + Height, X + Radius, Y + Height)
        gp.AddArc(X, Y + Height - (Radius * 2), Radius * 2, Radius * 2, 90, 90)
        gp.AddLine(X, Y + Height - (Radius * 2), X, Y + Radius)
        gp.AddArc(X, Y, Radius * 2, Radius * 2, 180, 90)
        gp.CloseFigure()
        Return gp
    End Function

    ''' <summary>
    ''' Ermittelt eine Mischfarbe aus zwei vorliegenden Color-Strukturen
    ''' </summary>
    ''' <param name="Col1">Color-Struktur</param>
    ''' <param name="Col2">Color-Struktur</param>
    ''' <returns>Color-Struktur</returns>
    Shared Function MidColor(ByVal Col1 As Color, ByVal Col2 As Color) As Color
        Dim A As Single = Math.Round(CSng(CInt(Col1.A) + CInt(Col2.A)) / 2.0F)
        Dim R As Single = Math.Round(CSng(CInt(Col1.R) + CInt(Col2.R)) / 2.0F)
        Dim G As Single = Math.Round(CSng(CInt(Col1.G) + CInt(Col2.G)) / 2.0F)
        Dim B As Single = Math.Round(CSng(CInt(Col1.B) + CInt(Col2.B)) / 2.0F)
        Return Color.FromArgb(A, R, G, B)
    End Function

    ''' <summary>
    ''' fllt einen GraphicsPath mit drei unterschiedlichen Brushes, um einen 3D-Effekt zu erreichen
    ''' </summary>
    ''' <param name="g">Graphics-Objekt</param>
    ''' <param name="GP">GraphicsPath, der die zu fllende Flche begrenzt</param>
    ''' <param name="NormalBrush">Normale Farbe</param>
    ''' <param name="LightBrush">hellere Farbe</param>
    ''' <param name="DarkBrush">dunklere Farbe</param>
    Shared Sub Fill3DPath(ByVal g As Graphics, ByVal GP As GraphicsPath, ByVal NormalBrush As Brush, ByVal LightBrush As Brush, ByVal DarkBrush As Brush)
        g.SmoothingMode = SmoothingMode.AntiAlias

        g.TranslateTransform(1.0F, 0.0F)
        g.FillPath(DarkBrush, GP)
        g.TranslateTransform(0.0F, 1.0F)
        g.FillPath(DarkBrush, GP)
        g.TranslateTransform(-1.0F, 0.0F)
        g.FillPath(DarkBrush, GP)
        g.TranslateTransform(-1.0F, -1.0F)
        g.FillPath(LightBrush, GP)
        g.TranslateTransform(0.0F, -1.0F)
        g.FillPath(LightBrush, GP)
        g.TranslateTransform(1.0F, 0.0F)
        g.FillPath(LightBrush, GP)
        g.TranslateTransform(0.0F, 1.0F)
        g.FillPath(NormalBrush, GP)

        g.SmoothingMode = SmoothingMode.Default
    End Sub
End Class
