''' <summary>
''' Zeichnet eine einfache ScrollBar mit rechteckigen Buttons
''' </summary>
Public Class BasicScrollBarPainter
    Inherits ScrollBarPainter

#Region " Private Fields "
    ' Farbensammlung zum Zeichnen der ScrollBar-Inhalte
    Private m_ScrollBarColors As ScrollBarColorCollection

    ' AreaPainter, die die einzelnen Areas auf der ScrollBar zeichnen
    Private m_AreaButtonTopPainter As ScrollBarAreaPainter
    Private m_AreaButtonBottomPainter As ScrollBarAreaPainter
    Private m_AreaLargeScrollTopPainter As ScrollBarAreaPainter
    Private m_AreaLargeScrollBottomPainter As ScrollBarAreaPainter
    Private m_AreaButtonScrollPainter As ScrollBarAreaPainter
    Private m_AreaBackgroundPainter As ScrollBarAreaPainter
#End Region

#Region " Constructor "
    Public Sub New()
        ' Farbensammlung und AreaPainter initialisieren
        m_ScrollBarColors = New ScrollBarColorCollection(Color.LightSkyBlue, Color.WhiteSmoke, Color.LightSteelBlue, Color.LightBlue)
        m_AreaButtonTopPainter = New ScrollBarButtonPainter(m_ScrollBarColors)
        m_AreaButtonBottomPainter = New ScrollBarButtonPainter(m_ScrollBarColors)
        m_AreaLargeScrollTopPainter = New ScrollBarBackAreaPainter(m_ScrollBarColors)
        m_AreaLargeScrollBottomPainter = New ScrollBarBackAreaPainter(m_ScrollBarColors)
        m_AreaButtonScrollPainter = New ScrollBarButtonPainter(m_ScrollBarColors)
        m_AreaBackgroundPainter = New ScrollBarBackAreaPainter(m_ScrollBarColors)
    End Sub
#End Region

#Region " Properties "

    ''' <summary>
    ''' AreaPainter fr den Hintergrund des Controls
    ''' </summary>
    Public Overrides ReadOnly Property AreaBackgroundPainter() As ScrollBarAreaPainter
        Get
            Return m_AreaBackgroundPainter
        End Get
    End Property

    ''' <summary>
    ''' AreaPainter fr den oberen Pfeil-Button
    ''' </summary>
    Public Overrides ReadOnly Property AreaButtonTopPainter() As ScrollBarAreaPainter
        Get
            Return m_AreaButtonTopPainter
        End Get
    End Property

    ''' <summary>
    ''' AreaPainter fr die obere Scroll-Flche
    ''' </summary>
    Public Overrides ReadOnly Property AreaLargeScrollTopPainter() As ScrollBarAreaPainter
        Get
            Return m_AreaLargeScrollTopPainter
        End Get
    End Property

    ''' <summary>
    ''' AreaPainter fr den Scroll-Button
    ''' </summary>
    Public Overrides ReadOnly Property AreaButtonScrollPainter() As ScrollBarAreaPainter
        Get
            Return m_AreaButtonScrollPainter
        End Get
    End Property

    ''' <summary>
    ''' AreaPainter fr den unteren Pfeil-Button
    ''' </summary>
    Public Overrides ReadOnly Property AreaButtonBottomPainter() As ScrollBarAreaPainter
        Get
            Return m_AreaButtonBottomPainter
        End Get
    End Property

    ''' <summary>
    ''' AreaPainter fr die untere Scroll-Flche
    ''' </summary>
    Public Overrides ReadOnly Property AreaLargeScrollBottomPainter() As ScrollBarAreaPainter
        Get
            Return m_AreaLargeScrollBottomPainter
        End Get
    End Property


    ''' <summary>
    ''' Lnge des oberen Pfeil-Buttons
    ''' </summary>
    Public Overrides ReadOnly Property ButtonTopLength() As Integer
        Get
            Return 17
        End Get
    End Property

    ''' <summary>
    ''' Lnge des unteren Pfeil-Buttons
    ''' </summary>
    Public Overrides ReadOnly Property ButtonBottomLength() As Integer
        Get
            Return 17
        End Get
    End Property

    ''' <summary>
    ''' Mindestlnge fr den Scroll-Button
    ''' </summary>
    ''' <value>Ein Wert > 0</value>
    Public Overrides ReadOnly Property MinButtonScrollLength() As Integer
        Get
            Return 17
        End Get
    End Property

    ''' <summary>
    ''' Minimum fr Breite
    ''' </summary>
    Public Overrides ReadOnly Property MinVirtualWidth() As Integer
        Get
            Return 17
        End Get
    End Property

    ''' <summary>
    ''' Standard-Farben der ScrollBar
    ''' </summary>
    Public Overrides ReadOnly Property ScrollBarColors() As ScrollBarColorCollection
        Get
            Return m_ScrollBarColors
        End Get
    End Property
#End Region

#Region " Private Classes "
    ''' <summary>
    ''' Zeichnet einen Button der BasicScrollBar
    ''' </summary>
    Private Class ScrollBarButtonPainter
        Inherits ScrollBarAreaPainter

#Region " Private Fields "
        ' Aktiver Brush zum Zeichnen der Flche
        Private m_ActiveBrush As SolidBrush
        ' Pen zum Zeichnen des Rahmens
        Private m_BorderPen As Pen
        ' Rechteck, das den Ramhmen festlegt
        Private m_BorderRect As Rectangle
        ' Farbensammlung des ScrollBarPainters
        Private WithEvents m_Colors As ScrollBarColorCollection
#End Region

#Region " Constructor "
        Public Sub New(ByVal Colors As ScrollBarColorCollection)
            ' Farbensammlung als Referenz bernehmen
            m_Colors = Colors
            ' Brushes und Pen definieren
            GetBrushes()
        End Sub
#End Region

#Region " Properties "

        ''' <summary>
        ''' Zeichenrechteck
        ''' </summary>
        ''' <remarks>Das Rechteck, das dieser AreaPainter als Flche zum Zeichnen bentigt. (Dieses 
        ''' Rechteck kann von dem der zugeordneten ScrollBar-Area abweichen.)</remarks>
        Public Overrides ReadOnly Property PaintRectangle() As System.Drawing.Rectangle
            Get
                Return Me.Area.Rectangle
            End Get
        End Property
#End Region

#Region " Methods "


        ''' <summary>
        ''' Eventhandler fr das Changed-Event der Farbensammlung
        ''' </summary>
        Private Sub m_Colors_Changed() Handles m_Colors.Changed
            ' legt Brushes und Pens zum Zeichnen nach ndern der Farbensammlung neu fest
            GetBrushes()
        End Sub

        ''' <summary>
        ''' Brushes und Pens neu festlegen
        ''' </summary>
        Private Sub GetBrushes()
            If Me.Area Is Nothing Then
                m_ActiveBrush = New SolidBrush(m_Colors.ForeColor)
            Else
                If Me.Area.Selected Then
                    m_ActiveBrush = New SolidBrush(m_Colors.SelectionColor)
                ElseIf Me.Area.Hoovered Then
                    m_ActiveBrush = New SolidBrush(m_Colors.HooverColor)
                Else
                    m_ActiveBrush = New SolidBrush(m_Colors.ForeColor)
                End If
            End If
            Dim Col As New HSLColor(m_ActiveBrush.Color)
            If Col.Saturation < 0.5R Then
                m_BorderPen = New Pen(ControlPaint.Dark(m_ActiveBrush.Color))
            Else
                m_BorderPen = New Pen(ControlPaint.LightLight(m_ActiveBrush.Color))
            End If

        End Sub

        ''' <summary>
        ''' tritt ein beim MouseDown-Ereignis ber der zugeordneten Area
        ''' </summary>
        Public Overrides Sub OnMouseDown()
            GetBrushes()
        End Sub

        ''' <summary>
        ''' tritt ein beim Eintreten der Maus in die zugeordnete Area
        ''' </summary>
        Public Overrides Sub OnMouseEnter()
            GetBrushes()
        End Sub

        ''' <summary>
        ''' tritt ein beim Verlassen der zugeordneten Area durch die Maus
        ''' </summary>
        Public Overrides Sub OnMouseLeave()
            GetBrushes()
        End Sub

        ''' <summary>
        ''' tritt ein beim MouseUp-Ereignis ber der zugeordneten Area
        ''' </summary>
        Public Overrides Sub OnMouseUp()
            GetBrushes()
        End Sub

        ''' <summary>
        ''' ermglicht das Aktualisieren nach nderungen der ScrollBarArea
        ''' </summary>
        Public Overrides Sub Update()
            Dim Original As Rectangle = Me.Area.Rectangle
            Dim Width As Integer = Original.Width - 3 'Math.Max(Original.Width - 3, 1)
            Dim Height As Integer = Original.Height - 3 'Math.Max(Original.Height - 3, 1)
            m_BorderRect = New Rectangle(Original.X + 1, Original.Y + 1, Width, Height)
        End Sub

        ''' <summary>
        ''' zeichnet die ScrollBarArea neu
        ''' </summary>
        Public Overrides Sub OnPaint(ByVal e As System.Windows.Forms.PaintEventArgs)
            e.Graphics.FillRectangle(m_ActiveBrush, Me.Area.Rectangle)
            e.Graphics.DrawRectangle(m_BorderPen, m_BorderRect)
        End Sub
#End Region

    End Class

    ''' <summary>
    ''' Zeichnet den Hintergrund der BasicScrollBar
    ''' </summary>
    Private Class ScrollBarBackAreaPainter
        Inherits ScrollBarAreaPainter

#Region " Private Fields "
        ' Aktiver Brush zum Zeichnen der Flche
        Private m_ActiveBrush As SolidBrush
        ' Farbensammlung des ScrollBarPainters
        Private WithEvents m_Colors As ScrollBarColorCollection
#End Region

#Region " Constructor "
        Public Sub New(ByVal Colors As ScrollBarColorCollection)
            m_Colors = Colors
            GetActiveBrush()
        End Sub
#End Region

#Region " Properties "

        ''' <summary>
        ''' Zeichenrechteck
        ''' </summary>
        ''' <remarks>Das Rechteck, das dieser AreaPainter als Flche zum Zeichnen bentigt. (Dieses 
        ''' Rechteck kann von dem der zugeordneten ScrollBar-Area abweichen.)</remarks>
        Public Overrides ReadOnly Property PaintRectangle() As System.Drawing.Rectangle
            Get
                Return Me.Area.Rectangle
            End Get
        End Property
#End Region

#Region " Methods "

        ''' <summary>
        ''' Eventhandler fr das Changed-Event der Farbensammlung
        ''' </summary>
        Private Sub m_Colors_Changed() Handles m_Colors.Changed
            ' legt Brushes und Pens zum Zeichnen nach ndern der Farbensammlung neu fest
            GetActiveBrush()
        End Sub

        ''' <summary>
        ''' legt den aktiven Brush neu fest
        ''' </summary>
        Private Sub GetActiveBrush()
            m_ActiveBrush = New SolidBrush(m_Colors.BackColor)
        End Sub

        ''' <summary>
        ''' tritt ein beim MouseDown-Ereignis ber der zugeordneten Area
        ''' </summary>
        Public Overrides Sub OnMouseDown()
        End Sub


        ''' <summary>
        ''' tritt ein beim Eintreten der Maus in die zugeordnete Area
        ''' </summary>
        Public Overrides Sub OnMouseEnter()
        End Sub

        ''' <summary>
        ''' tritt ein beim Verlassen der zugeordneten Area durch die Maus
        ''' </summary>
        Public Overrides Sub OnMouseLeave()
        End Sub

        ''' <summary>
        ''' tritt ein beim MouseUp-Ereignis ber der zugeordneten Area
        ''' </summary>
        Public Overrides Sub OnMouseUp()
        End Sub

        ''' <summary>
        ''' ermglicht das Aktualisieren nach nderungen der ScrollBarArea
        ''' </summary>
        Public Overrides Sub Update()

        End Sub

        ''' <summary>
        ''' zeichnet die ScrollBarArea neu
        ''' </summary>
        Public Overrides Sub OnPaint(ByVal e As System.Windows.Forms.PaintEventArgs)
            e.Graphics.FillRectangle(m_ActiveBrush, Me.Area.Rectangle)
        End Sub
#End Region

    End Class

#End Region

End Class
