Imports System.Drawing.Drawing2D
''' <summary>
''' Zeichnet einen Pfeil-Button mit berblend-Effekten fr die GlassScrollBar 
''' </summary>
Public Class GlassArrowButtonPainter
    Inherits GlassButtonPainter

    ' das Zeichnen des eigentlichen Buttons wird in der Basisklasse erledigt
    ' hier wird eigentlich nur noch der dreieckige Pfeil gezeichnet

#Region " Private Fields "

    ' Flag mit Info, ob es sich um den oberen bzw. linken Button handelt
    Private m_IsTopOrLeft As Boolean = False

    ' GraphicsPath fr den zu zeichnenden Pfeil
    Private m_ArrowPath As GraphicsPath

    ' Brush fr den zu zeichnenden Pfeil
    Private m_ArrowBrush As PathGradientBrush

    ' Pen fr den halbtransparente Rahmen um den Pfeil
    Private m_PathPen As Pen

#End Region

#Region " Constructor "

    ''' <summary>
    ''' Konstruktor fr den Glass-Pfeil-ButtonPainter
    ''' </summary>
    ''' <param name="Colors">Farbensammlung</param>
    ''' <param name="IsVisibleAtStart">Info, ob der Button beim Start sichtbar ist</param>
    ''' <param name="IsTopOrLeftArrow">Info, ob es sich um den oberen/linken Button handelt</param>
    Public Sub New(ByVal Colors As ScrollBarColorCollection, ByVal IsVisibleAtStart As Boolean, ByVal IsTopOrLeftArrow As Boolean)
        MyBase.New(Colors, IsVisibleAtStart)
        m_IsTopOrLeft = IsTopOrLeftArrow
        ' Pen fr den halbtransparente Rahmen um den Pfeil definieren
        m_PathPen = New Pen(Color.FromArgb(100, Color.White), 2.0F)
        m_PathPen.LineJoin = LineJoin.Round
    End Sub

#End Region

#Region " Methods "

    ''' <summary>
    ''' liefert ein Rechteck, auf dem der Button gezeichnet wird
    ''' </summary>
    Public Overrides Function GetButtonRectangle() As System.Drawing.Rectangle
        Dim Rect As Rectangle = Me.Area.Rectangle
        Dim TmpRect As New Rectangle(Rect.X, Rect.Y, Rect.Width - 1, Rect.Height - 1)
        TmpRect.Inflate(-1, -1)
        Return TmpRect
    End Function

    ''' <summary>
    ''' erstellt das auf der Oberflche des Buttons zu zeichnende Symbol neu
    ''' </summary>
    ''' <remarks>wird von der Basisklasse aus aufgerufen</remarks>
    Public Overrides Sub GetSymbol()
        ' neuer GraphicsPath fr den Pfeil
        m_ArrowPath = New GraphicsPath

        ' Mittelpunkt des Buttons
        Dim MidPoint As New PointF(CSng(m_ButtonRect.X) + CSng(m_ButtonRect.Width) / 2.0F, CSng(m_ButtonRect.Y) + CSng(m_ButtonRect.Height) / 2.0F)

        ' Gre fr den zuknftigen Pfeil
        Dim Size As SizeF
        If Me.Area.Orientation = Windows.Forms.Orientation.Vertical Then
            Size = New SizeF(8.0F, 4.5F)
        Else
            Size = New SizeF(4.5F, 8.0F)
        End If

        ' Rechteck, in dem der Pfeil gezeichnet werden soll
        Dim Rect As New RectangleF(Math.Round(MidPoint.X - Size.Width / 2.0F), Math.Round(MidPoint.Y - Size.Height / 2.0F), Size.Width, Size.Height)

        ' Eckpunkte fr den Pfeil-GraphicsPath
        Dim Pts(2) As PointF
        ' Eckfarben fr den Pfeil-Farbverlauf
        Dim Cols(2) As Color
        ' HSL-Instanz fr Basisfarbe des Pfeils erstellen
        ' die unterschiedlichen Farbschattierungen des Pfeils werden 
        ' durch nderung von Lightness erzeugt 
        Dim HSL As New HSLColor(m_SymbolWhite)

        ' CenterPoint fr den PathGradientBrush, mit dem der Pfeil gezeichnet wird
        Dim CenterPoint As PointF
        ' CenterColor fr den PathGradientBrush
        Dim CenterColor As Color

        ' Es gibt vier unterschiedliche Varianten, wie der Pfeil orientiert sein kann
        ' fr jede einzelne werden Pfad-Punkte und Farben separat definiert
        If Me.Area.Orientation = Windows.Forms.Orientation.Vertical Then
            If m_IsTopOrLeft Then
                ' Pfeil nach oben

                Pts(0) = New PointF(Rect.Left, Rect.Bottom)
                Pts(1) = New PointF(Rect.Left + Rect.Width / 2.0F, Rect.Top)
                Pts(2) = New PointF(Rect.Right, Rect.Bottom)
                Cols(0) = HSL.GetColorByLightness(33.0R / 100.0R)
                Cols(1) = HSL.GetColorByLightness(0.0R)
                Cols(2) = HSL.GetColorByLightness(71.0R / 100.0R)
                CenterPoint = New PointF(Rect.Left + Rect.Width / 2.0F + 1.0F, Rect.Bottom + 0.5F)
                CenterColor = HSL.GetColorByLightness(1.0R)
            Else
                ' Pfeil nach unten

                Rect.Offset(0.0F, 1.5F)

                Pts(0) = New PointF(Rect.Left, Rect.Top)
                Pts(1) = New PointF(Rect.Right, Rect.Top)
                Pts(2) = New PointF(Rect.Left + Rect.Width / 2.0F, Rect.Bottom)

                Cols(0) = HSL.GetColorByLightness(0.0R)
                Cols(1) = HSL.GetColorByLightness(55.0R / 100.0R)
                Cols(2) = HSL.GetColorByLightness(67.0R / 100.0R)

                CenterPoint = New PointF(Rect.Left + Rect.Width / 2.0F, Rect.Top - 0.5F)
                CenterColor = HSL.GetColorByLightness(0.0R)
            End If
        Else
            If m_IsTopOrLeft Then
                ' Pfeil nach links

                Rect.Offset(-0.5F, 0.0F)

                Pts(0) = New PointF(Rect.Left, Rect.Top + Rect.Height / 2.0F)
                Pts(1) = New PointF(Rect.Right, Rect.Top)
                Pts(2) = New PointF(Rect.Right, Rect.Bottom)

                Cols(0) = HSL.GetColorByLightness(0.0R)
                Cols(1) = HSL.GetColorByLightness(33.0R / 100.0R)
                Cols(2) = HSL.GetColorByLightness(71.0R / 100.0R)

                CenterPoint = New Point(Rect.Right, Rect.Bottom - Rect.Height / 3.0F)
                CenterColor = HSL.GetColorByLightness(1.0R)
            Else
                ' Pfeil nach rechts

                Rect.Offset(1.5F, 0.0F)

                Pts(0) = New PointF(Rect.Left, Rect.Top)
                Pts(1) = New PointF(Rect.Right, Rect.Top + Rect.Height / 2.0F)
                Pts(2) = New PointF(Rect.Left, Rect.Bottom)

                Cols(0) = HSL.GetColorByLightness(0.0R) 'Color.Black 
                Cols(1) = HSL.GetColorByLightness(67.0R / 100.0R)
                Cols(2) = HSL.GetColorByLightness(55.0R / 100.0R)

                CenterPoint = New Point(Rect.Right - Rect.Width / 3.0F, Rect.Bottom - Rect.Height / 4.0F)
                CenterColor = HSL.GetColorByLightness(1.0R) 'Color.White
            End If
        End If

        ' GraphicsPath fr Pfeil aus Eckpunkten erstellen
        m_ArrowPath.AddLines(Pts)

        ' PathgradientBrush zum Zeichnen des Pfeils erstellen
        Dim ArrowBrushPoints() As PointF
        Dim ArrowBrushCols() As Color
        If Me.Area.Orientation = Windows.Forms.Orientation.Vertical AndAlso Not m_IsTopOrLeft Then
            ReDim ArrowBrushPoints(3)
            ReDim ArrowBrushCols(3)
            ArrowBrushPoints(0) = Pts(0)
            ArrowBrushCols(0) = Cols(0)
            ArrowBrushPoints(1) = Pts(1)
            ArrowBrushCols(1) = Cols(1)
            ArrowBrushPoints(2) = New PointF(Rect.Right - Rect.Width / 4.0F + 0.5F, Rect.Bottom - Rect.Height / 3.0F)
            ArrowBrushCols(2) = HSL.GetColorByLightness(1.0R)
            ArrowBrushPoints(3) = Pts(2)
            ArrowBrushCols(3) = Cols(2)
        Else
            ArrowBrushPoints = Pts
            ArrowBrushCols = Cols
        End If

        m_ArrowBrush = New PathGradientBrush(ArrowBrushPoints)
        m_ArrowBrush.SurroundColors = ArrowBrushCols

        m_ArrowBrush.CenterPoint = CenterPoint
        m_ArrowBrush.CenterColor = CenterColor

        ' alle geffneten Figuren im GraphicsPath fr den Pfeil schlieen
        m_ArrowPath.CloseAllFigures()
    End Sub

    ''' <summary>
    ''' zeichnet das Vordergrund-Symbol (hier einen Pfeil) auf den Button
    ''' </summary>
    ''' <param name="e">PaintEventArgs</param>
    Public Overrides Sub PaintSymbol(ByVal e As System.Windows.Forms.PaintEventArgs)
        Dim g As Graphics = e.Graphics
        ' AntiAliasing aktivieren
        g.SmoothingMode = SmoothingMode.AntiAlias
        ' halbtransparenten Hintergrund-Rahmen zeichnen
        g.DrawPath(m_PathPen, m_ArrowPath)
        ' vorbergehend gar kein Antialiasing mehr verwenden
        ' (picelgenau zeichnen)
        g.SmoothingMode = SmoothingMode.None
        ' PfeilPfad wei ausfllen
        g.FillPath(Brushes.White, m_ArrowPath)
        ' Pfeil-Pfad mit PathgradientBrush fllen
        g.FillPath(m_ArrowBrush, m_ArrowPath)
        ' AntiAliasing auf Normalzustand zurcksetzen
        g.SmoothingMode = SmoothingMode.Default
    End Sub

#End Region

End Class
