Imports System.Drawing.Drawing2D
''' <summary>
''' Zeichnet einen Scroll-Button mit berblend-Effekten fr die GlassScrollBar 
''' </summary>
''' <remarks></remarks>
Public Class GlassScrollButtonPainter
    Inherits GlassButtonPainter

    ' das Zeichnen des eigentlichen Buttons wird in der Basisklasse erledigt
    ' hier wird eigentlich nur noch eine Art Griff-Symbol gezeichnet

#Region " Private Fields "

    ' Brush fr den Vordergrund des Griff-Symbols
    Private m_ColorBrush As LinearGradientBrush
    ' Brush fr den Schatten des Griff-Symbols
    Private m_ShadowBrush As LinearGradientBrush
    ' Rechteck, das den Rahmen fr ColorBrush bzw. ShadowBrush definiert
    Private m_BrushRect As Rectangle
    ' Rahmen-Reckteck fr das gesamte Griff-Symbol
    Private m_FullRect As Rectangle
    ' Brush zum Zeichnen des Hintergrunds fr das gesamte Griff-Symbol
    Private m_FullBrush As SolidBrush

    ' Flag - True wenn dass das Griff-Symbol wegen zu kleiner Button-Abmessungen
    ' nicht gezeichnet werden kann
    Private m_CanNotDraw As Boolean = True

#End Region

#Region " Constructor "

    ''' <summary>
    ''' Konstruktor fr den Glass-ScrollButton-Painter
    ''' </summary>
    ''' <param name="Colors">Farbensammlung</param>
    ''' <param name="IsVisibleAtStart">Info, ob der Button beim Start sichtbar ist</param>
    Public Sub New(ByVal Colors As ScrollBarColorCollection, ByVal IsVisibleAtStart As Boolean)
        MyBase.New(Colors, IsVisibleAtStart)
        ' Brush zum Zeichnen des Hintergrunds fr das Griff-Symbol
        m_FullBrush = New SolidBrush(Color.FromArgb(100, Color.White))
    End Sub

#End Region

#Region " Methods "

    ''' <summary>
    ''' liefert ein Rechteck, auf dem der Button gezeichnet wird
    ''' </summary>
    Public Overrides Function GetButtonRectangle() As System.Drawing.Rectangle
        Dim Rect As Rectangle = Me.Area.Rectangle
        Dim TmpRect As New Rectangle(Rect.X, Rect.Y, Rect.Width - 1, Rect.Height - 1)
        If Me.Area.Orientation = Windows.Forms.Orientation.Vertical Then
            TmpRect.Inflate(-1, 0)
        Else
            TmpRect.Inflate(0, -1)
        End If
        Return TmpRect
    End Function

    ''' <summary>
    ''' erstellt das auf der Oberflche des Buttons zu zeichnende Symbol neu
    ''' </summary>
    ''' <remarks>wird von der Basisklasse aus aufgerufen</remarks>
    Public Overrides Sub GetSymbol()

        ' Breite und Hhe fr das zuknftige Griff-Symbol
        ' Wenn das Griff-Symbol zu klein fr das Button-Rechteck
        ' wird, kann es nicht gezeichnet werden
        Dim ZielBreite, ZielHoehe As Integer
        If Me.Area.Orientation = Windows.Forms.Orientation.Vertical Then
            ZielBreite = 9
            ZielHoehe = 10
            If Me.Area.Rectangle.Height < 17 Then
                m_CanNotDraw = True
                Exit Sub
            Else
                m_CanNotDraw = False
            End If
        Else
            ZielBreite = 10
            ZielHoehe = 9
            If Me.Area.Rectangle.Width < 17 Then
                m_CanNotDraw = True
                Exit Sub
            Else
                m_CanNotDraw = False
            End If
        End If

        ' HSL-Instanz fr Basisfarbe des Griff-Symbols erstellen
        ' die unterschiedlichen Farbschattierungen des Pfeils werden 
        ' durch nderung von Lightness erzeugt 
        Dim HSL As New HSLColor(m_SymbolWhite)
        Dim Alpha As Integer = m_ActiveColor.A

        Dim ButtonGribColorStart As Color = Color.FromArgb(Alpha, HSL.GetColorByLightness(23.0R / 100.0R))
        Dim ButtonGribColorEnd As Color = Color.FromArgb(Alpha, HSL.GetColorByLightness(69.0R / 100.0R))

        Dim ButtonGribColorStart1 As Color = Color.FromArgb(Alpha, HSL.GetColorByLightness(40.0R / 100.0R))
        Dim ButtonGribColorEnd1 As Color = Color.FromArgb(Alpha, HSL.GetColorByLightness(79.0R / 100.0R))

        Dim ButtonGribColorMid As Color = SharedFunctions.MidColor(ButtonGribColorStart, ButtonGribColorEnd)
        Dim ButtonGribColorMid1 As Color = SharedFunctions.MidColor(ButtonGribColorStart1, ButtonGribColorEnd1)

        ' ColorBlend fr den Farbverlauf
        Dim myBlend As New ColorBlend
        myBlend.Positions = New Single() {0.0F, 0.3F, 0.6F, 1.0F}


        ' Rechteck, in dem das Griff-Symbol gezeichnet werden soll, ermitteln
        Dim Left As Integer = Me.Area.Rectangle.Left + (Me.Area.Rectangle.Width - ZielBreite) \ 2
        Dim Top As Integer = Me.Area.Rectangle.Top + (Me.Area.Rectangle.Height - ZielHoehe) \ 2
        m_FullRect = New Rectangle(Left, Top, ZielBreite, ZielHoehe)

        ' Abhngig von der Position dieses Rechtecks und der Orientierung der 
        ' ScrollBar werden die GradientBrushes neu definiert
        If Me.Area.Orientation = Windows.Forms.Orientation.Vertical Then
            m_BrushRect = New Rectangle(Left + 1, Top, ZielBreite - 2, 1)
            m_ColorBrush = New LinearGradientBrush(m_BrushRect, ButtonGribColorStart, ButtonGribColorEnd, LinearGradientMode.Horizontal)
            m_ShadowBrush = New LinearGradientBrush(m_BrushRect, ButtonGribColorStart1, ButtonGribColorEnd1, LinearGradientMode.Horizontal)
            m_FullRect.Offset(0, 1)
        Else
            m_BrushRect = New Rectangle(Left, Top + 1, 1, ZielHoehe - 2)
            m_ColorBrush = New LinearGradientBrush(m_BrushRect, ButtonGribColorStart, ButtonGribColorEnd, LinearGradientMode.Vertical)
            m_ShadowBrush = New LinearGradientBrush(m_BrushRect, ButtonGribColorStart1, ButtonGribColorEnd1, LinearGradientMode.Vertical)
            m_FullRect.Offset(1, 0)
        End If

        ' Interpolierungs-Farben fr Vordergrund-Brush bernehmen
        myBlend.Colors = New Color() {ButtonGribColorStart, ButtonGribColorMid, ButtonGribColorMid, ButtonGribColorEnd}
        m_ColorBrush.InterpolationColors = myBlend

        ' Interpolierungs-Farben fr Schatten-Brush bernehmen
        myBlend.Colors = New Color() {ButtonGribColorStart1, ButtonGribColorMid1, ButtonGribColorMid1, ButtonGribColorEnd1}
        m_ShadowBrush.InterpolationColors = myBlend

    End Sub

    ''' <summary>
    ''' zeichnet das Vordergrund-Symbol (hier ein Griff-Symbol) auf den Button
    ''' </summary>
    ''' <param name="e">PaintEventArgs</param>
    Public Overrides Sub PaintSymbol(ByVal e As System.Windows.Forms.PaintEventArgs)
        ' wenn das Symbol zu gro fr das Button-Rechteck ist, nichts zeichnen
        If m_CanNotDraw Then Exit Sub

        ' Referenz zum Graphics-Objekt der PaintEventArgs
        Dim g As Graphics = e.Graphics

        ' Rechteck zum Zeichnen eines Griff-Balkens
        Dim PaintRect As New Rectangle(m_BrushRect.X, m_BrushRect.Y, m_BrushRect.Width, m_BrushRect.Height)

        ' Hintergrund des Griffs mit halbtransparentem Brush fllen
        g.FillRectangle(m_FullBrush, m_FullRect)

        ' Smoothing ausschalten
        g.SmoothingMode = SmoothingMode.None

        ' drei Balken zeichnen, indem das Balken-Rechteck immer um ein paar
        ' Punkte verschoben wird
        If Me.Area.Orientation = Windows.Forms.Orientation.Vertical Then
            ' vertikale Orientierung
            For I As Integer = 0 To 2
                PaintRect.Offset(0, 2)
                ' Vordergrund zeichnen
                g.FillRectangle(m_ColorBrush, PaintRect)
                PaintRect.Offset(0, 1)
                ' Schatten zeichnen
                g.FillRectangle(m_ShadowBrush, PaintRect)
            Next
        Else
            ' horizontale Orientierung
            For I As Integer = 0 To 2
                PaintRect.Offset(2, 0)
                ' Vordergrund zeichnen
                g.FillRectangle(m_ColorBrush, PaintRect)
                PaintRect.Offset(1, 0)
                ' Schatten zeichnen
                g.FillRectangle(m_ShadowBrush, PaintRect)
            Next
        End If

        ' Smoothing auf default zurck setzen
        g.SmoothingMode = SmoothingMode.Default
    End Sub

#End Region

End Class
