''' <summary>
''' Diese Klasse verwaltet 20 verschiedene Farben fr einen Gradienten-hnlichen Farbverlauf
''' </summary>
''' <remarks></remarks>
Public Class GradientCollection

#Region " Konstruktoren "

    ''' <summary>
    ''' Konstruktor mit einer Start- und einer End-Farbe
    ''' </summary>
    ''' <param name="StartColor">Start-Farbe</param>
    ''' <param name="EndColor">End-Farbe</param>
    Public Sub New(ByVal StartColor As Color, ByVal EndColor As Color)
        ReDim m_Colors(19)
        Dim ADist, RDist, GDist, BDist As Decimal

        ADist = CDec(CInt(EndColor.A) - CInt(StartColor.A)) / 20D
        RDist = CDec(CInt(EndColor.R) - CInt(StartColor.R)) / 20D
        GDist = CDec(CInt(EndColor.G) - CInt(StartColor.G)) / 20D
        BDist = CDec(CInt(EndColor.B) - CInt(StartColor.B)) / 20D

        For I As Integer = 1 To 18
            Dim Alpha As Integer = Math.Min(StartColor.A + ADist * CDec(I), 255)
            Dim Red As Integer = Math.Min(StartColor.R + RDist * CDec(I), 255)
            Dim Green As Integer = Math.Min(StartColor.G + GDist * CDec(I), 255)
            Dim Blue As Integer = Math.Min(StartColor.B + BDist * CDec(I), 255)
            m_Colors(I) = Color.FromArgb(Alpha, Red, Green, Blue)
        Next

        m_Colors(0) = StartColor
        m_Colors(19) = EndColor
    End Sub

    ''' <summary>
    ''' Konstruktor mit einer Start-, Mittel- und einer End-Farbe
    ''' </summary>
    ''' <param name="StartColor">Start-Farbe</param>
    ''' <param name="MiddleColor">Mittel-Farbe</param>
    ''' <param name="EndColor">End-Farbe</param>
    Public Sub New(ByVal StartColor As Color, ByVal MiddleColor As Color, ByVal EndColor As Color)
        ReDim m_Colors(19)

        'Colors(12) = MiddleColor

        Dim ADist, RDist, GDist, BDist As Decimal

        ADist = CDec(CInt(MiddleColor.A) - CInt(StartColor.A)) / 13D
        RDist = CDec(CInt(MiddleColor.R) - CInt(StartColor.R)) / 13D
        GDist = CDec(CInt(MiddleColor.G) - CInt(StartColor.G)) / 13D
        BDist = CDec(CInt(MiddleColor.B) - CInt(StartColor.B)) / 13D

        For I As Integer = 1 To 11
            Dim Alpha As Integer = Math.Min(StartColor.A + ADist * CDec(I), 255)
            Dim Red As Integer = Math.Min(StartColor.R + RDist * CDec(I), 255)
            Dim Green As Integer = Math.Min(StartColor.G + GDist * CDec(I), 255)
            Dim Blue As Integer = Math.Min(StartColor.B + BDist * CDec(I), 255)
            m_Colors(I) = Color.FromArgb(Alpha, Red, Green, Blue)
        Next

        ADist = CDec(CInt(EndColor.A) - CInt(MiddleColor.A)) / 7D
        RDist = CDec(CInt(EndColor.R) - CInt(MiddleColor.R)) / 7D
        GDist = CDec(CInt(EndColor.G) - CInt(MiddleColor.G)) / 7D
        BDist = CDec(CInt(EndColor.B) - CInt(MiddleColor.B)) / 7D

        For I As Integer = 1 To 6
            Dim Alpha As Integer = Math.Min(MiddleColor.A + ADist * CDec(I), 255)
            Dim Red As Integer = Math.Min(MiddleColor.R + RDist * CDec(I), 255)
            Dim Green As Integer = Math.Min(MiddleColor.G + GDist * CDec(I), 255)
            Dim Blue As Integer = Math.Min(MiddleColor.B + BDist * CDec(I), 255)
            m_Colors(12 + I) = Color.FromArgb(Alpha, Red, Green, Blue)
        Next


        m_Colors(0) = StartColor
        m_Colors(12) = MiddleColor
        m_Colors(19) = EndColor
    End Sub

#End Region

#Region " Private Felder "

    ' 20 verschiedene Farben (als Farbverlauf) sind in diesem Array enthalten
    Private m_Colors() As Color
#End Region

#Region " Properties "

    ''' <summary>
    ''' gibt eine einzelne von 20 verschiedenen Farben des Farb-Verlaufs zurck
    ''' </summary>
    ''' <param name="Index">Farb-Index (0 bis 19)</param>
    ''' <returns>Color-Struktur</returns>
    ReadOnly Property Colors(ByVal Index As Integer) As Color
        Get
            If Index > 19 Then
                Throw New Exception
            End If
            Return m_Colors(Index)
        End Get
    End Property
#End Region

End Class