''' <summary>
''' Zeichnet einen Pfeil-Button in einem PictureScrollBarPainter
''' </summary>
Public Class ArrowPictureButton
    Inherits ScrollBarAreaPainter

#Region " Private Fields "

    ' enthlt drei Bitmaps zum Zeichnen dieser Area
    Private m_Images As SelectionBitmapSource
    ' Zeichenrechteck
    Private m_PaintRectangle As Rectangle
    ' enthlt Info, ob es sich um den oberen bzw linken Pfeil-Button handelt
    Private m_IsTop As Boolean = False
    ' Referenz zur Bitmap-Quelle
    Private m_PicturePainterSource As PicturePainterSource

#End Region

#Region " Constructor "
    Public Sub New(ByVal PicturePainterSource As PicturePainterSource, ByVal Images As SelectionBitmapSource, ByVal IsTop As Boolean)
        m_PicturePainterSource = PicturePainterSource
        m_Images = Images
        m_IsTop = IsTop
    End Sub
#End Region

#Region " Properties "

    ''' <summary>
    ''' Es gibt drei Mglichkeiten fr SelectionState: Normal, Gehoovert, Selektiert
    ''' </summary>
    Private WriteOnly Property SelectionState() As SelectionStates
        Set(ByVal value As SelectionStates)
            m_Images.SelectionState = value
        End Set
    End Property

    ''' <summary>
    ''' Zeichenrechteck
    ''' </summary>
    ''' <remarks> Das Rechteck, das dieser AreaPainter als Flche zum Zeichnen bentigt. (Dieses 
    ''' Rechteck kann von dem der zugeordneten ScrollBar-Area abweichen.)</remarks>
    Public Overrides ReadOnly Property PaintRectangle() As System.Drawing.Rectangle
        Get
            Return m_PaintRectangle
        End Get
    End Property
#End Region

#Region " Methods "

    ''' <summary>
    ''' ermglicht das Aktualisieren nach nderungen der ScrollBarArea
    ''' </summary>
    Public Overrides Sub Update()
        m_PicturePainterSource.SetOrientation(Me.Area.Orientation)
        SetPaintRect()
    End Sub

    ''' <summary>
    ''' tritt ein beim Eintreten der Maus in die zugeordnete Area
    ''' </summary>
    Public Overrides Sub OnMouseEnter()
        SelectionState = SelectionStates.Hoovered
    End Sub

    ''' <summary>
    ''' tritt ein beim Verlassen der zugeordneten Area durch die Maus
    ''' </summary>
    Public Overrides Sub OnMouseLeave()
        SelectionState = SelectionStates.Normal
    End Sub

    ''' <summary>
    ''' tritt ein beim MouseDown-Ereignis ber der zugeordneten Area
    ''' </summary>
    Public Overrides Sub OnMouseDown()
        SelectionState = SelectionStates.Selected
    End Sub

    ''' <summary>
    ''' tritt ein beim MouseUp-Ereignis ber der zugeordneten Area
    ''' </summary>
    Public Overrides Sub OnMouseUp()
        SelectionState = SelectionStates.Hoovered
    End Sub

    ''' <summary>
    ''' legt das Zeichenrechteck nach nderungen der ScrollBar-Area oder der Orientierung neu fest
    ''' </summary>
    Private Sub SetPaintRect()
        Dim Left, Top As Integer
        Dim Rect As Rectangle = Me.Area.Rectangle
        Dim PaintSize As Size = m_Images.ActiveBitmap.Size

        If Me.Area.Orientation = Windows.Forms.Orientation.Vertical Then
            Left = Rect.X + (Rect.Width - PaintSize.Width) \ 2
            If m_IsTop Then
                Top = 0
            Else
                Top = Rect.Bottom - PaintSize.Height
            End If
        Else
            Top = Rect.Y + (Rect.Height - PaintSize.Height) \ 2
            If m_IsTop Then
                Left = 0
            Else
                Left = Rect.Right - PaintSize.Width
            End If
        End If

        m_PaintRectangle = New Rectangle(New Point(Left, Top), PaintSize)
    End Sub

    ''' <summary>
    ''' zeichnet die ScrollBarArea neu
    ''' </summary>
    Public Overrides Sub OnPaint(ByVal e As System.Windows.Forms.PaintEventArgs)
        Dim g As Graphics = e.Graphics
        g.DrawImage(m_Images.ActiveBitmap, m_PaintRectangle)
    End Sub
#End Region

End Class
