''' <summary>
''' definiert Standard-Werte und Basis-Bilder fr einen PictureScrollBarPainter
''' </summary>
''' <remarks></remarks>
Public MustInherit Class PicturePainterSource

#Region " Fields "

    ' Basis-Grafiken zum Zeichnen der PictureScrollBar
    ' in dreifacher Ausfhrung (fr normalen, gehooverten und selektierten Zustand)

    ' Grafiken zum Zeichnen des oberen Pfeil-Buttons
    Public ButtonTopImages As New SelectionBitmapSource

    ' Grafiken zum Zeichnen des ScrollButtons
    Public ScrollTopImages As New SelectionBitmapSource
    Public ScrollMiddleImages As New SelectionBitmapSource
    Public ScrollBottomImages As New SelectionBitmapSource

    ' Grafiken zum Zeichnen des unteren Pfeil-Buttons
    Public ButtonBottomImages As New SelectionBitmapSource


    ' Grafik als Hintergrund fr die Scroll-Flche
    Public ScrollAreaBackImage As Bitmap

    ' Flag, das die Orientierung der ScrollBar enthlt,
    ' um nderungen der ScrollBar feststellen zu knnen
    Private m_Orientation As Orientation = Orientation.Horizontal

#End Region

#Region " Properties "
    ''' <summary>
    ''' Bildvorlage, die die ScrollBar in normalem Zustand zeigt 
    ''' </summary>
    MustOverride ReadOnly Property NormalImage() As Image

    ''' <summary>
    ''' Bildvorlage, die die ScrollBar in gehoovertem Zustand zeigt 
    ''' </summary>
    MustOverride ReadOnly Property HooverImage() As Image

    ''' <summary>
    ''' Bildvorlage, die die ScrollBar in selektiertem Zustand zeigt 
    ''' </summary>
    MustOverride ReadOnly Property SelectedImage() As Image

    ''' <summary>
    ''' Vertikaler Bereich auf der Bildvorlage, der den oberen Pfeil-Button enthlt
    ''' </summary>
    MustOverride ReadOnly Property ButtonTopRange() As Range

    ''' <summary>
    ''' Vertikaler Bereich auf der Bildvorlage, der den oberen Abschnitt des Scroll-Buttons enthlt
    ''' </summary>
    MustOverride ReadOnly Property ScrollTopRange() As Range

    ''' <summary>
    ''' Vertikaler Bereich auf der Bildvorlage, der den mittleren Abschnitt des Scroll-Buttons enthlt
    ''' </summary>
    MustOverride ReadOnly Property ScrollMiddleRange() As Range

    ''' <summary>
    ''' Vertikaler Bereich auf der Bildvorlage, der den unteren Abschnitt des Scroll-Buttons enthlt
    ''' </summary>
    MustOverride ReadOnly Property ScrollBottomRange() As Range

    ''' <summary>
    ''' Vertikaler Bereich auf der Bildvorlage, der den unteren Pfeil-Button enthlt
    ''' </summary>
    MustOverride ReadOnly Property ButtonBottomRange() As Range

    ''' <summary>
    ''' Vertikaler Bereich auf der Bildvorlage, der den Hintergrund fr den Scrollbereich enthlt
    ''' </summary>
    MustOverride ReadOnly Property ScrollAreaBackgroundRange() As Range


    ''' <summary>
    ''' berschneidung zwischen Scroll-Button und oberem Pfeil-Button
    ''' </summary>
    MustOverride ReadOnly Property ButtonTopOverlap() As Integer

    ''' <summary>
    ''' berschneidung zwischen Scroll-Button und oberem Pfeil-Button
    ''' </summary>
    MustOverride ReadOnly Property ButtonBottomOverlap() As Integer

    ''' <summary>
    ''' minimale Lnge des Scroll-Button
    ''' </summary>
    MustOverride ReadOnly Property ButtonScrollMinLength() As Integer
#End Region

#Region " Methods "

    ''' <summary>
    ''' ndert die in den SelectionBitmapSources enthaltenen Images
    ''' abhngig von der Orientierung der ScrollBar
    ''' </summary>
    Public Sub SetOrientation(ByVal Orientation As Orientation)
        ' Methode nur ausfhren, wenn die verlangte Orientierung wirklich neu ist
        If m_Orientation <> Orientation Then
            m_Orientation = Orientation
        Else
            Exit Sub
        End If

        ' die drei Bitmap-Vorlagen in einem Array zusammenfassen
        Dim Originals(2) As Bitmap
        Originals(0) = NormalImage
        Originals(1) = HooverImage
        Originals(2) = SelectedImage

        ' alle SelectionBitmapSources mit den entsprechend ausgerichteten 
        ' Bildausschnitten versorgen
        RecreateBitmapSource(ButtonTopImages, Originals, ButtonTopRange)
        RecreateBitmapSource(ButtonBottomImages, Originals, ButtonBottomRange)

        RecreateBitmapSource(ScrollTopImages, Originals, ScrollTopRange)
        RecreateBitmapSource(ScrollMiddleImages, Originals, ScrollMiddleRange)
        RecreateBitmapSource(ScrollBottomImages, Originals, ScrollBottomRange)

        Dim Rect As New Rectangle(0, ScrollAreaBackgroundRange.Position, NormalImage.Width, ScrollAreaBackgroundRange.Length)

        ' Hintergrund fr die ScrollArea (Area zwischen oberem und unterem Pfeil-Button)
        ScrollAreaBackImage = CreateBitmap(NormalImage, Rect)
    End Sub

    ''' <summary>
    ''' erstellt Bitmaps fr eine SelectionBitmapSource neu
    ''' </summary>
    Private Sub RecreateBitmapSource(ByVal BitmapSource As SelectionBitmapSource, ByVal Originals() As Bitmap, ByVal Rng As Range)
        Dim Rect As New Rectangle(0, Rng.Position, Originals(0).Width, Rng.Length)
        BitmapSource.SetBitmaps(CreateBitmap(Originals(0), Rect), CreateBitmap(Originals(1), Rect), CreateBitmap(Originals(2), Rect))
    End Sub

    ''' <summary>
    ''' Liefert ein  Bitmap als Bildausschnitt
    ''' </summary>
    Private Function CreateBitmap(ByVal Original As Bitmap, ByVal Rect As Rectangle) As Bitmap
        ' neue Bitmap mit gewunschter Gre und 32bpp PixelFormat erzeugen
        Dim Result As New Bitmap(Rect.Width, Rect.Height, Drawing.Imaging.PixelFormat.Format32bppArgb)
        ' Graphics-Objekt fr diese Bitmap erhalten
        Dim g As Graphics = Graphics.FromImage(Result)
        ' auf dieses Graphics-Objekt den Bildausschnitt aus dem Original-Bitmap zeichnen
        g.DrawImage(Original, New Rectangle(New Point(0, 0), Rect.Size), Rect, GraphicsUnit.Pixel)

        ' wenn die Orientierung horizontal ist, das Bild um 270 Grad drehen
        ' und um die y-Achse spiegeln
        If m_Orientation = Orientation.Horizontal Then
            Result.RotateFlip(RotateFlipType.Rotate270FlipY)
        End If

        Return Result
    End Function
#End Region

End Class
