''' <summary>
''' Painter fr die Hintergrund-Area einer PictureScrollBar
''' </summary>
Public Class PictureScrollBarBackGroundPainter
    Inherits ScrollBarAreaPainter
#Region " Private Fields "
    Private m_ActiveBrush As SolidBrush
    ' Farbensammlung des ScrollBarPainters
    Private WithEvents m_Colors As ScrollBarColorCollection
    ' Flag, enthlt Info, ob die Flche gehoovert ist
    Private m_Hoovered As Boolean = False
    ' letzte bekannte Orientierung
    Private m_KnownOrientation As Orientation = Orientation.Horizontal
    ' letzte bekannte Gre der ScrollBar-Area
    Private m_KnownSize As New Size(0, 0)
    ' Bitmap, das den zu zeichnenden Inhalt enthlt
    Private m_Bitmap As Bitmap
    ' Position des zu zeichnenden Inhalts relativ zur ScrollBar-Area
    Private m_BitmapShift As New Size(0, 0)
    ' Gre des zu zeichnenden Inhalts (als Flche)
    Private m_BitmapSize As Size
    ' Referenz zur Bitmap-Quelle
    Private m_PicturePainterSource As PicturePainterSource
    ' Zeichenrechteck
    Private m_PaintRectangle As Rectangle
    ' es kann der Fall eintreten, dass kein Hintergrund-Bitmap sichtbar ist
    Private m_BitmapVisible As Boolean = False
#End Region

#Region " Constructor "
    Public Sub New(ByVal PicturePainterSource As PicturePainterSource, ByVal Colors As ScrollBarColorCollection)
        m_PicturePainterSource = PicturePainterSource
        m_Colors = Colors
        GetActiveBrush()
    End Sub
#End Region

#Region " Properties "

    ''' <summary>
    ''' Zeichenrechteck
    ''' </summary>
    ''' <remarks>Das Rechteck, das dieser AreaPainter als Flche zum Zeichnen bentigt. (Dieses 
    ''' Rechteck kann von dem der zugeordneten ScrollBar-Area abweichen.)</remarks>
    Public Overrides ReadOnly Property PaintRectangle() As System.Drawing.Rectangle
        Get
            Return Me.Area.Rectangle
        End Get
    End Property
#End Region

#Region " Methods "

    ''' <summary>
    ''' Eventhandler fr das Changed-Event der Farbensammlung
    ''' </summary>
    Private Sub m_Colors_Changed() Handles m_Colors.Changed
        ' aktiven Brush neu festlegen
        GetActiveBrush()
    End Sub

    ''' <summary>
    ''' legt den aktuellen Brush zum Zeichnen der Flche neu fest
    ''' </summary>
    Private Sub GetActiveBrush()
        If m_Hoovered Then
            m_ActiveBrush = New SolidBrush(ControlPaint.LightLight(m_Colors.BackColor))
        Else
            m_ActiveBrush = New SolidBrush(m_Colors.BackColor)
        End If
    End Sub

    ''' <summary>
    ''' tritt ein beim MouseDown-Ereignis ber der zugeordneten Area
    ''' </summary>
    Public Overrides Sub OnMouseDown()

    End Sub

    ''' <summary>
    ''' tritt ein beim Eintreten der Maus in die zugeordnete Area
    ''' </summary>
    Public Overrides Sub OnMouseEnter()
        If m_Hoovered <> True Then
            m_Hoovered = True
            GetActiveBrush()
        End If
    End Sub

    ''' <summary>
    ''' tritt ein beim Verlassen der zugeordneten Area durch die Maus
    ''' </summary>
    Public Overrides Sub OnMouseLeave()
        If m_Hoovered <> False Then
            m_Hoovered = False
            GetActiveBrush()
        End If
    End Sub

    ''' <summary>
    ''' tritt ein beim MouseUp-Ereignis ber der zugeordneten Area
    ''' </summary>
    Public Overrides Sub OnMouseUp()

    End Sub

    ''' <summary>
    ''' zeichnet die ScrollBarArea neu
    ''' </summary>
    Public Overrides Sub OnPaint(ByVal e As System.Windows.Forms.PaintEventArgs)
        e.Graphics.FillRectangle(m_ActiveBrush, e.ClipRectangle)
        If m_BitmapVisible Then e.Graphics.DrawImage(m_Bitmap, m_PaintRectangle)
    End Sub

    ''' <summary>
    ''' ermglicht das Aktualisieren nach nderungen der ScrollBarArea
    ''' </summary>
    Public Overrides Sub Update()
        ' Falls notwendig, Bildausschnitte zum Zeichnen der ScrollBar 
        ' mit neuer Orientierung neu erstellen
        m_PicturePainterSource.SetOrientation(Me.Area.Orientation)

        ' nur wenn Orientierung oder Gre der Flche sich gendert haben,
        ' wird die Bitmap zum Zeichnen neu definiert
        If m_KnownOrientation <> Me.Area.Orientation OrElse m_KnownSize <> Me.Area.Rectangle.Size Then
            m_KnownOrientation = Me.Area.Orientation
            m_KnownSize = Me.Area.Rectangle.Size
            RecreateBitmap()
        End If

        ' Das Rechteck, in dem die Bitmap gezeichnet werden soll, neu feststellen
        Dim Rect As Rectangle = Me.Area.Rectangle
        m_PaintRectangle = New Rectangle(Rect.X + m_BitmapShift.Width, Rect.Y + m_BitmapShift.Height, m_BitmapSize.Width, m_BitmapSize.Height)

    End Sub

    ''' <summary>
    ''' erstellt die Hintergrund-Bitmap neu
    ''' </summary>
    Private Sub RecreateBitmap()
        Dim Rect As Rectangle = Me.Area.Rectangle
        Dim Img As Image = m_PicturePainterSource.ScrollAreaBackImage

        Dim Width, Height As Integer
        If Me.Area.Orientation = Orientation.Vertical Then
            Width = Img.Width
            Height = Me.Area.Rectangle.Height - m_PicturePainterSource.ButtonTopRange.Length - m_PicturePainterSource.ButtonBottomRange.Length

            ' Ermitteln, ob tatschlich eine Bitmap erstellt werden muss
            If Width < 1 OrElse Height < 1 Then
                m_BitmapVisible = False
                Exit Sub
            Else
                m_BitmapVisible = True
            End If

            m_BitmapSize = New Size(Width, Height)
            m_Bitmap = New Bitmap(Width, Height, Drawing.Imaging.PixelFormat.Format32bppArgb)
            Dim g As Graphics = Graphics.FromImage(m_Bitmap)

            ' auf die Bitmap wird zeilenweise die Quell-Bitmap gezeichnet
            Dim TmpRect As New Rectangle(0, 0, Width, 1)
            For I As Integer = 0 To Height - 1
                g.DrawImage(Img, TmpRect)
                TmpRect.Offset(0, 1)
            Next

        Else
            Height = Img.Height
            Width = Me.Area.Rectangle.Width - m_PicturePainterSource.ButtonTopRange.Length - m_PicturePainterSource.ButtonBottomRange.Length

            ' Ermitteln, ob tatschlich eine Bitmap erstellt werden muss
            If Width < 1 OrElse Height < 1 Then
                m_BitmapVisible = False
                Exit Sub
            Else
                m_BitmapVisible = True
            End If

            m_BitmapSize = New Size(Width, Height)


            m_Bitmap = New Bitmap(Width, Height, Drawing.Imaging.PixelFormat.Format32bppArgb)
            Dim g As Graphics = Graphics.FromImage(m_Bitmap)

            ' auf die Bitmap wird spaltenweise die Quell-Bitmap gezeichnet
            Dim TmpRect As New Rectangle(0, 0, 1, Height)
            For I As Integer = 0 To Width - 1
                g.DrawImage(Img, TmpRect)
                TmpRect.Offset(1, 0)
            Next
        End If

        ' Bitmap-Shift ermitteln
        If Me.Area.Orientation = Orientation.Vertical Then
            m_BitmapShift = New Size((Rect.Width - m_BitmapSize.Width) \ 2, m_PicturePainterSource.ButtonTopRange.Length)
        Else
            m_BitmapShift = New Size(m_PicturePainterSource.ButtonTopRange.Length, (Rect.Height - m_BitmapSize.Height) \ 2)
        End If
    End Sub
#End Region

End Class
