''' <summary>
''' Painter fr eine CustomScrollBar - zeichnet ScrollBar-Elemente 
''' als Bildausschnitte einer vorliegenden ScrollBar-Grafik
''' </summary>
Public Class PictureScrollBarPainter
    Inherits ScrollBarPainter

#Region " Private Fields "
    ' Farbensammlung zum Zeichnen der ScrollBar-Inhalte
    Private m_ScrollBarColors As ScrollBarColorCollection

    ' AreaPainter, die die einzelnen Areas auf der ScrollBar zeichnen
    Private m_AreaButtonTopPainter As ArrowPictureButton
    Private m_AreaLargeScrollTopPainter As EmptyScrollBarAreaPainter
    Private m_AreaButtonScrollPainter As ScrollPictureButton
    Private m_AreaLargeScrollBottomPainter As EmptyScrollBarAreaPainter
    Private m_AreaButtonBottomPainter As ArrowPictureButton
    Private m_AreaBackgroundPainter As PictureScrollBarBackGroundPainter

    Private m_Source As PicturePainterSource
#End Region

#Region " Constructor "
    Public Sub New(ByVal Source As PicturePainterSource)
        m_Source = Source

        ' Farbensammlung und AreaPainter initialisieren
        m_ScrollBarColors = New ScrollBarColorCollection(Color.LightSkyBlue, Color.WhiteSmoke, Color.LightSteelBlue, Color.LightBlue)

        m_AreaButtonTopPainter = New ArrowPictureButton(m_Source, m_Source.ButtonTopImages, True)
        m_AreaLargeScrollTopPainter = New EmptyScrollBarAreaPainter
        m_AreaButtonScrollPainter = New ScrollPictureButton(m_Source, m_Source.ScrollTopImages, m_Source.ScrollMiddleImages, m_Source.ScrollBottomImages, m_Source.ButtonTopOverlap, m_Source.ButtonBottomOverlap)
        m_AreaLargeScrollBottomPainter = New EmptyScrollBarAreaPainter
        m_AreaButtonBottomPainter = New ArrowPictureButton(m_Source, m_Source.ButtonBottomImages, False)
        m_AreaBackgroundPainter = New PictureScrollBarBackGroundPainter(m_Source, m_ScrollBarColors)
    End Sub
#End Region

#Region " Properties "

    ''' <summary>
    ''' AreaPainter fr den Hintergrund des Controls
    ''' </summary>
    Public Overrides ReadOnly Property AreaBackgroundPainter() As ScrollBarAreaPainter
        Get
            Return m_AreaBackgroundPainter
        End Get
    End Property

    ''' <summary>
    ''' AreaPainter fr den oberen Pfeil-Button
    ''' </summary>
    Public Overrides ReadOnly Property AreaButtonTopPainter() As ScrollBarAreaPainter
        Get
            Return m_AreaButtonTopPainter
        End Get
    End Property

    ''' <summary>
    ''' AreaPainter fr die obere Scroll-Flche
    ''' </summary>
    Public Overrides ReadOnly Property AreaLargeScrollTopPainter() As ScrollBarAreaPainter
        Get
            Return m_AreaLargeScrollTopPainter
        End Get
    End Property

    ''' <summary>
    ''' AreaPainter fr den Scroll-Button
    ''' </summary>
    Public Overrides ReadOnly Property AreaButtonScrollPainter() As ScrollBarAreaPainter
        Get
            Return m_AreaButtonScrollPainter
        End Get
    End Property
    ''' <summary>
    ''' AreaPainter fr den unteren Pfeil-Button
    ''' </summary>
    Public Overrides ReadOnly Property AreaButtonBottomPainter() As ScrollBarAreaPainter
        Get
            Return m_AreaButtonBottomPainter
        End Get
    End Property

    ''' <summary>
    ''' AreaPainter fr die untere Scroll-Flche
    ''' </summary>
    Public Overrides ReadOnly Property AreaLargeScrollBottomPainter() As ScrollBarAreaPainter
        Get
            Return m_AreaLargeScrollBottomPainter
        End Get
    End Property


    ''' <summary>
    ''' Lnge des oberen Pfeil-Buttons
    ''' </summary>
    Public Overrides ReadOnly Property ButtonTopLength() As Integer
        Get
            Return m_Source.ButtonTopRange.Length '- m_Source.ButtonTopOverlap
        End Get
    End Property

    ''' <summary>
    ''' Lnge des unteren Pfeil-Buttons
    ''' </summary>
    Public Overrides ReadOnly Property ButtonBottomLength() As Integer
        Get
            Return m_Source.ButtonBottomRange.Length - m_Source.ButtonBottomOverlap
        End Get
    End Property

    ''' <summary>
    ''' Mindestlnge fr den Scroll-Button
    ''' </summary>
    ''' <value>Ein Wert > 0</value>
    Public Overrides ReadOnly Property MinButtonScrollLength() As Integer
        Get
            Return Math.Max(m_Source.ScrollTopRange.Length + m_Source.ScrollMiddleRange.Length + m_Source.ScrollBottomRange.Length, m_Source.ButtonScrollMinLength)
        End Get
    End Property

    ''' <summary>
    ''' Minimum fr Breite
    ''' </summary>
    Public Overrides ReadOnly Property MinVirtualWidth() As Integer
        Get
            Return m_Source.NormalImage.Width
        End Get
    End Property

    ''' <summary>
    ''' Standard-Farben der ScrollBar
    ''' </summary>
    Public Overrides ReadOnly Property ScrollBarColors() As ScrollBarColorCollection
        Get
            Return m_ScrollBarColors
        End Get
    End Property
#End Region

End Class
