''' <summary>
''' zeichnet die ScrollButton-Area einer PictureScrollBar
''' </summary>
''' <remarks></remarks>
Public Class ScrollPictureButton
    Inherits ScrollBarAreaPainter

#Region " Private Fields "
    ' enthlt drei Bitmaps zum Zeichnen des oberen Abschnitts des ScrollButtons
    Private m_TopImages As SelectionBitmapSource
    ' enthlt drei Bitmaps zum Zeichnen des mittleren Abschnitts des ScrollButtons
    Private m_MiddleImages As SelectionBitmapSource
    ' enthlt drei Bitmaps zum Zeichnen des unteren Abschnitts des ScrollButtons
    Private m_BottomImages As SelectionBitmapSource

    ' Bitmap, die den zusammengesetzten Button enthlt
    Private m_Bitmap As Bitmap

    ' berlappung zwischen oberem Pfeil-Button und ScrollButton
    Private m_TopOverlap As Integer
    ' berlappung zwischen unterem Pfeil-Button und ScrollButton
    Private m_BottomOverlap As Integer

    ' Zeichenrechteck
    Private m_PaintRectangle As Rectangle
    ' Referenz zur Bitmap-Quelle
    Private m_PicturePainterSource As PicturePainterSource

    ' letzte bekannte Orientierung der ScrollBar-Area
    Private m_KnownOrientation As Orientation = Orientation.Horizontal
    ' letzte bekannte Gre der ScrollBar-Area
    Private m_KnownSize As New Size(0, 0)
    ' Gre der zu zeichnenden Bitmap
    Private m_BitmapSize As Size
    ' relative Position der Bitmap zum Rechteck der ScrollBar-Area
    Private m_BitmapShift As Size

    ' aktueller Maus-Auswahl-Status
    Private m_SelectionState As SelectionStates = SelectionStates.Normal
#End Region

#Region " Constructor "
    Public Sub New(ByVal PicturePainterSource As PicturePainterSource, ByVal TopImages As SelectionBitmapSource, ByVal MiddleImages As SelectionBitmapSource, ByVal BottomImages As SelectionBitmapSource, ByVal TopOverlap As Integer, ByVal BottomOverlap As Integer)
        m_PicturePainterSource = PicturePainterSource

        m_TopImages = TopImages
        m_MiddleImages = MiddleImages
        m_BottomImages = BottomImages

        m_TopOverlap = TopOverlap
        m_BottomOverlap = BottomOverlap
    End Sub
#End Region

#Region " Properties "
    Private WriteOnly Property SelectionState() As SelectionStates
        Set(ByVal value As SelectionStates)
            ' nur wenn SelectionState sich ndert
            If m_SelectionState <> value Then
                m_SelectionState = value
                ' SelectionState an alle drei BitmapSourcen bergeben
                m_TopImages.SelectionState = value
                m_MiddleImages.SelectionState = value
                m_BottomImages.SelectionState = value
                ' Bitmap neu erstellen
                RecreateBitmap()
            End If
        End Set
    End Property

    ''' <summary>
    ''' Zeichenrechteck
    ''' </summary>
    ''' <remarks>Das Rechteck, das dieser AreaPainter als Flche zum Zeichnen bentigt. (Dieses 
    ''' Rechteck kann von dem der zugeordneten ScrollBar-Area abweichen.)</remarks>
    Public Overrides ReadOnly Property PaintRectangle() As System.Drawing.Rectangle
        Get
            Return m_PaintRectangle
        End Get
    End Property
#End Region

#Region " Methods "

    ''' <summary>
    ''' ermglicht das Aktualisieren nach nderungen der ScrollBarArea
    ''' </summary>
    Public Overrides Sub Update()
        ' falls die Orientierung gendert wurde, werden alle Bildausschnitte der
        ' PicturePainterSource neu definiert
        m_PicturePainterSource.SetOrientation(Me.Area.Orientation)

        ' nur wenn sichOrientierung oder Gre der Flche gendert haben,
        ' wird die Bitmap neu berechnet
        If m_KnownOrientation <> Me.Area.Orientation OrElse m_KnownSize <> Me.Area.Rectangle.Size Then
            ' aktuelle Orientierung und Gre der Flche fr nchsten Durchlauf dieser
            ' Methode festhalten
            m_KnownOrientation = Me.Area.Orientation
            m_KnownSize = Me.Area.Rectangle.Size
            ' Bitmap neu erstellen
            RecreateBitmap()
        End If

        ' Zeichenrechteck neu festlegen
        Dim Rect As Rectangle = Me.Area.Rectangle
        m_PaintRectangle = New Rectangle(Rect.X + m_BitmapShift.Width, Rect.Y + m_BitmapShift.Height, m_BitmapSize.Width, m_BitmapSize.Height)
    End Sub

    ''' <summary>
    ''' erstellt die zu zeichnende Bitmap neu
    ''' </summary>
    Private Sub RecreateBitmap()
        Dim Rect As Rectangle = Me.Area.Rectangle

        Dim TopImage As Image = m_TopImages.ActiveBitmap
        Dim MiddleImage As Image = m_MiddleImages.ActiveBitmap
        Dim BottomImage As Image = m_BottomImages.ActiveBitmap

        Dim Width, Height As Integer
        If Me.Area.Orientation = Orientation.Vertical Then
            Width = TopImage.Width
            Height = Me.Area.Rectangle.Height + m_TopOverlap

            m_Bitmap = New Bitmap(Width, Height, Drawing.Imaging.PixelFormat.Format32bppArgb)
            Dim g As Graphics = Graphics.FromImage(m_Bitmap)

            g.DrawImage(TopImage, New Rectangle(New Point(0, 0), TopImage.Size))

            Dim TmpRect As New Rectangle(0, TopImage.Height, Width, 1)
            For I As Integer = 0 To Height - BottomImage.Height - TopImage.Height - 1
                g.DrawImage(MiddleImage, TmpRect)
                TmpRect.Offset(0, 1)
            Next

            g.DrawImage(BottomImage, New Rectangle(New Point(0, Height - BottomImage.Height), BottomImage.Size))
            m_BitmapSize = New Size(Width, Height)
        Else
            Height = TopImage.Height
            Width = Me.Area.Rectangle.Width + m_TopOverlap

            m_Bitmap = New Bitmap(Width, Height, Drawing.Imaging.PixelFormat.Format32bppArgb)
            Dim g As Graphics = Graphics.FromImage(m_Bitmap)

            g.DrawImage(TopImage, New Rectangle(New Point(0, 0), TopImage.Size))

            Dim TmpRect As New Rectangle(TopImage.Width, 0, 1, Height)
            For I As Integer = 0 To Width - BottomImage.Width - TopImage.Width - 1
                g.DrawImage(MiddleImage, TmpRect)
                TmpRect.Offset(1, 0)
            Next

            g.DrawImage(BottomImage, New Rectangle(New Point(Width - BottomImage.Width, 0), BottomImage.Size))
            m_BitmapSize = New Size(Width, Height)
        End If


        If Me.Area.Orientation = Orientation.Vertical Then
            m_BitmapShift = New Size((Rect.Width - m_BitmapSize.Width) \ 2, m_TopOverlap * -1)
        Else
            m_BitmapShift = New Size(m_TopOverlap * -1, (Rect.Height - m_BitmapSize.Height) \ 2)
        End If

    End Sub


    ''' <summary>
    ''' tritt ein beim Eintreten der Maus in die zugeordnete Area
    ''' </summary>
    Public Overrides Sub OnMouseEnter()
        SelectionState = SelectionStates.Hoovered
    End Sub

    ''' <summary>
    ''' tritt ein beim Verlassen der zugeordneten Area durch die Maus
    ''' </summary>
    Public Overrides Sub OnMouseLeave()
        SelectionState = SelectionStates.Normal
    End Sub

    ''' <summary>
    ''' tritt ein beim MouseDown-Ereignis ber der zugeordneten Area
    ''' </summary>
    Public Overrides Sub OnMouseDown()
        SelectionState = SelectionStates.Selected
    End Sub

    ''' <summary>
    ''' tritt ein beim MouseUp-Ereignis ber der zugeordneten Area
    ''' </summary>
    Public Overrides Sub OnMouseUp()
        SelectionState = SelectionStates.Hoovered
    End Sub

    ''' <summary>
    ''' zeichnet die ScrollBarArea neu
    ''' </summary>
    Public Overrides Sub OnPaint(ByVal e As System.Windows.Forms.PaintEventArgs)
        e.Graphics.DrawImage(m_Bitmap, m_PaintRectangle)
    End Sub

#End Region

End Class
