Imports System.Drawing.Drawing2D
''' <summary>
''' zeichnet eine ScrollBar mit abgerundeten farbigen Buttons
''' </summary>
Public Class SimpleScrollBarPainter
    Inherits ScrollBarPainter

#Region " Private Fields "
    ' Farbensammlung zum Zeichnen der ScrollBar-Inhalte
    Private WithEvents m_ScrollBarColors As ScrollBarColorCollection

    ' AreaPainter, die die einzelnen Areas auf der ScrollBar zeichnen
    Private m_AreaButtonTopPainter As ScrollBarAreaPainter
    Private m_AreaButtonBottomPainter As ScrollBarAreaPainter
    Private m_AreaLargeScrollTopPainter As ScrollBarAreaPainter
    Private m_AreaLargeScrollBottomPainter As ScrollBarAreaPainter
    Private m_AreaButtonScrollPainter As ScrollBarAreaPainter
    Private WithEvents m_AreaBackgroundPainter As BackgroundPainter

    Private m_ScrollBarEnabled As Boolean = True
    Private m_TriBrushes As TriStateBrushCollection
#End Region

#Region " Constructor "
    Public Sub New()

        ' Farbensammlung und AreaPainter initialisieren
        m_ScrollBarColors = New ScrollBarColorCollection(Color.Orange, Color.WhiteSmoke, Color.Red, Color.LightBlue)
        m_TriBrushes = New TriStateBrushCollection
        RefreshBrushes()

        m_AreaButtonTopPainter = New TopDownButtonPainter(m_TriBrushes, True)
        m_AreaButtonBottomPainter = New TopDownButtonPainter(m_TriBrushes, False)
        m_AreaLargeScrollTopPainter = New LargeScrollPainter(m_TriBrushes)
        m_AreaLargeScrollBottomPainter = New LargeScrollPainter(m_TriBrushes)
        m_AreaButtonScrollPainter = New ScrollButtonPainter(m_TriBrushes)
        m_AreaBackgroundPainter = New BackgroundPainter(m_TriBrushes)
    End Sub
#End Region

#Region " Properties "

    ''' <summary>
    ''' AreaPainter fr den Hintergrund des Controls
    ''' </summary>
    Public Overrides ReadOnly Property AreaBackgroundPainter() As ScrollBarAreaPainter
        Get
            Return m_AreaBackgroundPainter
        End Get
    End Property

    ''' <summary>
    ''' AreaPainter fr den oberen Pfeil-Button
    ''' </summary>
    Public Overrides ReadOnly Property AreaButtonTopPainter() As ScrollBarAreaPainter
        Get
            Return m_AreaButtonTopPainter
        End Get
    End Property

    ''' <summary>
    ''' AreaPainter fr die obere Scroll-Flche
    ''' </summary>
    Public Overrides ReadOnly Property AreaLargeScrollTopPainter() As ScrollBarAreaPainter
        Get
            Return m_AreaLargeScrollTopPainter
        End Get
    End Property

    ''' <summary>
    ''' AreaPainter fr den Scroll-Button
    ''' </summary>
    Public Overrides ReadOnly Property AreaButtonScrollPainter() As ScrollBarAreaPainter
        Get
            Return m_AreaButtonScrollPainter
        End Get
    End Property
    ''' <summary>
    ''' AreaPainter fr den unteren Pfeil-Button
    ''' </summary>
    Public Overrides ReadOnly Property AreaButtonBottomPainter() As ScrollBarAreaPainter
        Get
            Return m_AreaButtonBottomPainter
        End Get
    End Property

    ''' <summary>
    ''' AreaPainter fr die untere Scroll-Flche
    ''' </summary>
    Public Overrides ReadOnly Property AreaLargeScrollBottomPainter() As ScrollBarAreaPainter
        Get
            Return m_AreaLargeScrollBottomPainter
        End Get
    End Property


    ''' <summary>
    ''' Lnge des oberen Pfeil-Buttons
    ''' </summary>
    Public Overrides ReadOnly Property ButtonTopLength() As Integer
        Get
            Return 17
        End Get
    End Property

    ''' <summary>
    ''' Lnge des unteren Pfeil-Buttons
    ''' </summary>
    Public Overrides ReadOnly Property ButtonBottomLength() As Integer
        Get
            Return 17
        End Get
    End Property

    ''' <summary>
    ''' Mindestlnge fr den Scroll-Button
    ''' </summary>
    ''' <value>Ein Wert > 0</value>
    Public Overrides ReadOnly Property MinButtonScrollLength() As Integer
        Get
            Return 17
        End Get
    End Property

    ''' <summary>
    ''' Minimum fr Breite
    ''' </summary>
    Public Overrides ReadOnly Property MinVirtualWidth() As Integer
        Get
            Return 17
        End Get
    End Property

    ''' <summary>
    ''' Standard-Farben der ScrollBar
    ''' </summary>
    Public Overrides ReadOnly Property ScrollBarColors() As ScrollBarColorCollection
        Get
            Return m_ScrollBarColors
        End Get
    End Property
#End Region

#Region " Methods "
    ''' <summary>
    ''' aktualisiert die Brushes zum Zeichnen der SimpleScrollBar
    ''' </summary>
    Private Sub RefreshBrushes()
        Dim BackColor, ForeColor, HooverColor, SelectionColor As Color
        If m_ScrollBarEnabled Then
            BackColor = m_ScrollBarColors.BackColor
            ForeColor = m_ScrollBarColors.ForeColor
            HooverColor = m_ScrollBarColors.HooverColor
            SelectionColor = m_ScrollBarColors.SelectionColor
        Else
            BackColor = SharedFunctions.MidColor(m_ScrollBarColors.BackColor, SystemColors.Control)
            ForeColor = SharedFunctions.MidColor(m_ScrollBarColors.ForeColor, SystemColors.Control)
            HooverColor = SharedFunctions.MidColor(m_ScrollBarColors.HooverColor, SystemColors.Control)
            SelectionColor = SharedFunctions.MidColor(m_ScrollBarColors.SelectionColor, SystemColors.Control)
        End If

        m_TriBrushes.BackGroundBrushes.Brush.Color = BackColor
        m_TriBrushes.BackGroundBrushes.BrushLight.Color = ControlPaint.Light(BackColor)
        m_TriBrushes.BackGroundBrushes.BrushDark.Color = ControlPaint.Dark(BackColor)

        m_TriBrushes.NormalBrushes.Brush.Color = ForeColor
        m_TriBrushes.NormalBrushes.BrushLight.Color = ControlPaint.Light(ForeColor)
        m_TriBrushes.NormalBrushes.BrushDark.Color = ControlPaint.Dark(ForeColor)

        m_TriBrushes.HooverBrushes.Brush.Color = HooverColor
        m_TriBrushes.HooverBrushes.BrushLight.Color = ControlPaint.Light(HooverColor)
        m_TriBrushes.HooverBrushes.BrushDark.Color = ControlPaint.Dark(HooverColor)

        m_TriBrushes.SelectionBrushes.Brush.Color = SelectionColor
        m_TriBrushes.SelectionBrushes.BrushLight.Color = ControlPaint.Light(SelectionColor)
        m_TriBrushes.SelectionBrushes.BrushDark.Color = ControlPaint.Dark(SelectionColor)
    End Sub

    ''' <summary>
    ''' Eventhandler fr das EnabledChanged-Event des AreaBackgroundPainters
    ''' </summary>
    ''' <param name="Enabled">Aktivierungszustand der ScrollBar</param>
    Private Sub m_AreaBackgroundPainter_EnabledChanged(ByVal Enabled As Boolean) Handles m_AreaBackgroundPainter.EnabledChanged
        m_ScrollBarEnabled = Enabled
        ' Brushes und Pens neu festlegen
        RefreshBrushes()
        ' (Invalidate erfolgt durch das ScrollBar-Control)
    End Sub

    ''' <summary>
    ''' Eventhandler fr das Changed-Event der Farbensammlung
    ''' </summary>
    ''' <remarks></remarks>
    Private Sub m_ScrollBarColors_Changed() Handles m_ScrollBarColors.Changed
        RefreshBrushes()
        ' (Invalidate erfolgt durch das ScrollBar-Control)
    End Sub
 
#End Region

#Region " Private Classes "
    ''' <summary>
    ''' zeichnet den ScrollButton auf einer SimpleScrollBar
    ''' </summary>
    Private Class ScrollButtonPainter
        Inherits ScrollBarAreaPainter

#Region " Private Fields "
        Private m_TriBrushes As TriStateBrushCollection
        Private m_Path As GraphicsPath
        Private m_Brushes As BrushCollection
#End Region
#Region " Constructor "
        Public Sub New(ByVal TriBrushes As TriStateBrushCollection)
            m_TriBrushes = TriBrushes
            m_Brushes = m_TriBrushes.NormalBrushes
        End Sub
#End Region
#Region " Properties "

        ''' <summary>
        ''' Zeichenrechteck
        ''' </summary>
        ''' <remarks>Das Rechteck, das dieser AreaPainter als Flche zum Zeichnen bentigt. (Dieses 
        ''' Rechteck kann von dem der zugeordneten ScrollBar-Area abweichen.)</remarks>
        Public Overrides ReadOnly Property PaintRectangle() As System.Drawing.Rectangle
            Get
                Return Me.Area.Rectangle
            End Get
        End Property
#End Region
#Region " Methods "

        ''' <summary>
        ''' tritt ein beim MouseDown-Ereignis ber der zugeordneten Area
        ''' </summary>
        Public Overrides Sub OnMouseDown()
            m_Brushes = m_TriBrushes.SelectionBrushes
        End Sub

        ''' <summary>
        ''' tritt ein beim Eintreten der Maus in die zugeordnete Area
        ''' </summary>
        Public Overrides Sub OnMouseEnter()
            m_Brushes = m_TriBrushes.HooverBrushes
        End Sub

        ''' <summary>
        ''' tritt ein beim Verlassen der zugeordneten Area durch die Maus
        ''' </summary>
        Public Overrides Sub OnMouseLeave()
            m_Brushes = m_TriBrushes.NormalBrushes
        End Sub

        ''' <summary>
        ''' tritt ein beim MouseUp-Ereignis ber der zugeordneten Area
        ''' </summary>
        Public Overrides Sub OnMouseUp()
            m_Brushes = m_TriBrushes.HooverBrushes
        End Sub

        ''' <summary>
        ''' ermglicht das Aktualisieren nach nderungen der ScrollBarArea
        ''' </summary>
        Public Overrides Sub Update()
            GetPath()
        End Sub
        Private Sub GetPath()
            Dim AreaRect As Rectangle = Me.Area.Rectangle
            Dim Rect As New RectangleF(AreaRect.X, AreaRect.Y, AreaRect.Width - 1, AreaRect.Height - 1)
            If Me.Area.Orientation = Orientation.Vertical Then
                Rect.Inflate(-2.0F, -1.0F)
            Else
                Rect.Inflate(-1.0F, -2.0F)
            End If
            ' runden GraphicsPath ermitteln
            m_Path = SharedFunctions.GetRoundPath(Rect.X, Rect.Y, Rect.Width, Rect.Height, 2.0F)
        End Sub

        ''' <summary>
        ''' zeichnet die ScrollBarArea neu
        ''' </summary>
        Public Overrides Sub OnPaint(ByVal e As System.Windows.Forms.PaintEventArgs)
            ' den abgerundeten GraphicsPath mit den aktuellen Brushes neu zeichnen
            SharedFunctions.Fill3DPath(e.Graphics, m_Path, m_Brushes.Brush, m_Brushes.BrushLight, m_Brushes.BrushDark)
        End Sub

#End Region

    End Class

    ''' <summary>
    ''' zeichnet den PfeilButton einer SimpleScrollBar
    ''' </summary>
    Private Class TopDownButtonPainter
        Inherits ScrollBarAreaPainter

#Region " Private Fields "
        ' alle verfgbaren Brushes fr verschiedene Selection-States
        Private m_TriBrushes As TriStateBrushCollection
        ' aktive Brushes
        Private m_ActiveBrushes As BrushCollection
        ' GraphicsPath, der einen Pfeil darstellt
        Private m_Path As GraphicsPath
        ' Ausrichtung des Pfeils
        Private m_ArrowDirection As ArrowDirection
        ' enthlt Info, ob es sich um den oberen bzw linken Pfeil-Button handelt
        Private m_IsTop As Boolean = False
#End Region
#Region " Constructor "
        Public Sub New(ByVal TriBrushes As TriStateBrushCollection, ByVal IsTop As Boolean)
            m_TriBrushes = TriBrushes
            m_ActiveBrushes = m_TriBrushes.NormalBrushes
            m_IsTop = IsTop
        End Sub
#End Region
#Region " Properties "

        ''' <summary>
        ''' Zeichenrechteck
        ''' </summary>
        ''' <remarks>Das Rechteck, das dieser AreaPainter als Flche zum Zeichnen bentigt. (Dieses 
        ''' Rechteck kann von dem der zugeordneten ScrollBar-Area abweichen.)</remarks>
        Public Overrides ReadOnly Property PaintRectangle() As System.Drawing.Rectangle
            Get
                Return Me.Area.Rectangle
            End Get
        End Property
#End Region
#Region " Methods "

        ''' <summary>
        ''' tritt ein beim MouseDown-Ereignis ber der zugeordneten Area
        ''' </summary>
        Public Overrides Sub OnMouseDown()
            m_ActiveBrushes = m_TriBrushes.SelectionBrushes
        End Sub


        ''' <summary>
        ''' tritt ein beim Eintreten der Maus in die zugeordnete Area
        ''' </summary>
        Public Overrides Sub OnMouseEnter()
            m_ActiveBrushes = m_TriBrushes.HooverBrushes
        End Sub

        ''' <summary>
        ''' tritt ein beim Verlassen der zugeordneten Area durch die Maus
        ''' </summary>
        Public Overrides Sub OnMouseLeave()
            m_ActiveBrushes = m_TriBrushes.NormalBrushes
        End Sub

        ''' <summary>
        ''' tritt ein beim MouseUp-Ereignis ber der zugeordneten Area
        ''' </summary>
        Public Overrides Sub OnMouseUp()
            m_ActiveBrushes = m_TriBrushes.HooverBrushes
        End Sub

        ''' <summary>
        ''' ermglicht das Aktualisieren nach nderungen der ScrollBarArea
        ''' </summary>
        Public Overrides Sub Update()
            If m_IsTop Then
                If Me.Area.Orientation = Orientation.Vertical Then
                    m_ArrowDirection = ArrowDirections.North
                Else
                    m_ArrowDirection = ArrowDirections.West
                End If
            Else
                If Me.Area.Orientation = Orientation.Vertical Then
                    m_ArrowDirection = ArrowDirections.South
                Else
                    m_ArrowDirection = ArrowDirections.East
                End If
            End If
            ' GraphicsPath fr den zu zeichnenden Pfeil neu ermitteln
            GetPath()
        End Sub
        Private Sub GetPath()
            Dim pathRect As RectangleF
            Dim Rect As Rectangle = Me.Area.Rectangle
            If m_IsTop Then
                If Me.Area.Orientation = Orientation.Vertical Then
                    pathRect = New RectangleF(Rect.X, Rect.Y, Rect.Width - 1, Rect.Height - 2)
                Else
                    pathRect = New RectangleF(Rect.X, Rect.Y, Rect.Width - 2, Rect.Height - 1)
                End If
            Else
                If Me.Area.Orientation = Orientation.Vertical Then
                    pathRect = New RectangleF(Rect.X, Rect.Y + 1, Rect.Width - 1, Rect.Height - 2)
                Else
                    pathRect = New RectangleF(Rect.X + 1, Rect.Y, Rect.Width - 2, Rect.Height - 1)
                End If
            End If
            pathRect.Inflate(-1.0F, -1.0F)
            m_Path = GetArrowPath(pathRect, m_ArrowDirection)
        End Sub

        ''' <summary>
        ''' zeichnet die ScrollBarArea neu
        ''' </summary>
        Public Overrides Sub OnPaint(ByVal e As System.Windows.Forms.PaintEventArgs)
            SharedFunctions.Fill3DPath(e.Graphics, m_Path, m_ActiveBrushes.Brush, m_ActiveBrushes.BrushLight, m_ActiveBrushes.BrushDark)
        End Sub
        ''' <summary>
        ''' liefert einen GraphicsPath fr einen Pfeil fr eine der vier mglichen Ausrichtungen
        ''' </summary>
        Private Function GetArrowPath(ByVal Rect As RectangleF, ByVal Direction As ArrowDirections)

            Dim gp As New GraphicsPath(Drawing2D.FillMode.Alternate)
            Dim Pts(2) As PointF
            Select Case Direction
                Case ArrowDirections.South
                    Pts(0) = New Point(Rect.X, Rect.Y)
                    Pts(1) = New Point(Rect.Right, Rect.Y)
                    Pts(2) = New Point(Rect.X + Rect.Width / 2.0F, Rect.Bottom)
                Case ArrowDirections.East
                    Pts(0) = New Point(Rect.X, Rect.Y)
                    Pts(1) = New Point(Rect.Right, Rect.Y + Rect.Height / 2.0F)
                    Pts(2) = New Point(Rect.X, Rect.Bottom)
                Case ArrowDirections.North
                    Pts(0) = New Point(Rect.X, Rect.Bottom)
                    Pts(1) = New Point(Rect.X + Rect.Width / 2.0F, Rect.Y)
                    Pts(2) = New Point(Rect.Right, Rect.Bottom)
                Case ArrowDirections.West
                    Pts(0) = New Point(Rect.X, Rect.Y + Rect.Height / 2.0F)
                    Pts(1) = New Point(Rect.Right, Rect.Y)
                    Pts(2) = New Point(Rect.Right, Rect.Bottom)
            End Select
            gp.AddPolygon(Pts)
            gp.CloseFigure()
            Return gp
        End Function
#End Region

        Private Enum ArrowDirections
            North
            East
            South
            West
        End Enum

    End Class

    ''' <summary>
    ''' zeichnet die LargeScroll-Area einer SimpleScrollBar
    ''' </summary>
    Private Class LargeScrollPainter
        Inherits ScrollBarAreaPainter

#Region " Private Fields "
        Private m_TriBrushes As TriStateBrushCollection
        ' Aktiver Brush zum Zeichnen der Flche
        Private m_ActiveBrush As SolidBrush
#End Region
#Region " Constructor "
        Public Sub New(ByVal TriBrushes As TriStateBrushCollection)
            m_TriBrushes = TriBrushes
            m_ActiveBrush = m_TriBrushes.BackGroundBrushes.Brush
        End Sub
#End Region
#Region " Properties "

        ''' <summary>
        ''' Zeichenrechteck
        ''' </summary>
        ''' <remarks>Das Rechteck, das dieser AreaPainter als Flche zum Zeichnen bentigt. (Dieses 
        ''' Rechteck kann von dem der zugeordneten ScrollBar-Area abweichen.)</remarks>
        Public Overrides ReadOnly Property PaintRectangle() As System.Drawing.Rectangle
            Get
                Return Me.Area.Rectangle
            End Get
        End Property
#End Region
#Region " Methods "

        ''' <summary>
        ''' tritt ein beim MouseDown-Ereignis ber der zugeordneten Area
        ''' </summary>
        Public Overrides Sub OnMouseDown()
            m_ActiveBrush = m_TriBrushes.BackGroundBrushes.BrushDark
        End Sub

        ''' <summary>
        ''' tritt ein beim Eintreten der Maus in die zugeordnete Area
        ''' </summary>
        Public Overrides Sub OnMouseEnter()
            m_ActiveBrush = m_TriBrushes.BackGroundBrushes.BrushLight
        End Sub

        ''' <summary>
        ''' tritt ein beim Verlassen der zugeordneten Area durch die Maus
        ''' </summary>
        Public Overrides Sub OnMouseLeave()
            m_ActiveBrush = m_TriBrushes.BackGroundBrushes.Brush
        End Sub

        ''' <summary>
        ''' tritt ein beim MouseUp-Ereignis ber der zugeordneten Area
        ''' </summary>
        Public Overrides Sub OnMouseUp()
            m_ActiveBrush = m_TriBrushes.BackGroundBrushes.BrushLight
        End Sub

        ''' <summary>
        ''' ermglicht das Aktualisieren nach nderungen der ScrollBarArea
        ''' </summary>
        Public Overrides Sub Update()

        End Sub

        ''' <summary>
        ''' zeichnet die ScrollBarArea neu
        ''' </summary>
        Public Overrides Sub OnPaint(ByVal e As System.Windows.Forms.PaintEventArgs)
            e.Graphics.FillRectangle(m_ActiveBrush, Me.Area.Rectangle)
        End Sub
#End Region


    End Class

    ''' <summary>
    ''' zeichnet den Hintergrund einer SimpleScrollBar
    ''' </summary>
    Private Class BackgroundPainter
        Inherits ScrollBarAreaPainter

#Region " Private Fields "
        Private m_TriBrushes As TriStateBrushCollection
        ' Aktiver Brush zum Zeichnen der Flche
        Private m_ActiveBrush As SolidBrush

        ' hlt als Flag den Aktivierungszustand der ScrollBar
        Private m_Enabled As Boolean = True

        ''' <summary>
        ''' Tritt ein, wenn sich der Aktivierungszustand der ScrollBar ndert
        ''' </summary>
        ''' <param name="Enabled"></param>
        Public Event EnabledChanged(ByVal Enabled As Boolean)
#End Region

#Region " Constructor "
        Public Sub New(ByVal TriBrushes As TriStateBrushCollection)
            m_TriBrushes = TriBrushes
            m_ActiveBrush = m_TriBrushes.BackGroundBrushes.Brush
        End Sub
#End Region

#Region " Properties "

        ''' <summary>
        ''' Zeichenrechteck
        ''' </summary>
        ''' <remarks>Das Rechteck, das dieser AreaPainter als Flche zum Zeichnen bentigt. (Dieses 
        ''' Rechteck kann von dem der zugeordneten ScrollBar-Area abweichen.)</remarks>
        Public Overrides ReadOnly Property PaintRectangle() As System.Drawing.Rectangle
            Get
                Return Me.Area.Rectangle
            End Get
        End Property
#End Region

#Region " Methods "

        ''' <summary>
        ''' tritt ein beim MouseDown-Ereignis ber der zugeordneten Area
        ''' </summary>
        Public Overrides Sub OnMouseDown()

        End Sub

        ''' <summary>
        ''' tritt ein beim Eintreten der Maus in die zugeordnete Area
        ''' </summary>
        Public Overrides Sub OnMouseEnter()

        End Sub

        ''' <summary>
        ''' tritt ein beim Verlassen der zugeordneten Area durch die Maus
        ''' </summary>
        Public Overrides Sub OnMouseLeave()

        End Sub

        ''' <summary>
        ''' tritt ein beim MouseUp-Ereignis ber der zugeordneten Area
        ''' </summary>
        Public Overrides Sub OnMouseUp()

        End Sub

        ''' <summary>
        ''' ermglicht das Aktualisieren nach nderungen der ScrollBarArea
        ''' </summary>
        Public Overrides Sub Update()
            If m_Enabled <> Me.Area.Enabled Then
                m_Enabled = Me.Area.Enabled
                RaiseEvent EnabledChanged(m_Enabled)
            End If
        End Sub

        ''' <summary>
        ''' zeichnet die ScrollBarArea neu
        ''' </summary>
        Public Overrides Sub OnPaint(ByVal e As System.Windows.Forms.PaintEventArgs)
            e.Graphics.FillRectangle(m_ActiveBrush, Me.Area.Rectangle)
        End Sub
#End Region

    End Class

    ''' <summary>
    ''' enthlt 12 Brushes 
    ''' </summary>
    Private Class TriStateBrushCollection
        Public NormalBrushes As New BrushCollection
        Public HooverBrushes As New BrushCollection
        Public SelectionBrushes As New BrushCollection
        Public BackGroundBrushes As New BrushCollection
    End Class

    ''' <summary>
    ''' enthlt 3 Brushes
    ''' </summary>
    Private Class BrushCollection
        Public Brush As New SolidBrush(Color.White)
        Public BrushLight As New SolidBrush(Color.White)
        Public BrushDark As New SolidBrush(Color.White)
    End Class
#End Region


End Class
